@@ -3,7 +3,7 @@
  * 
  *
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -26,10 +26,13 @@ CommandMap cmdMap[] =
   { "arg",           CMD_LI },
   { "attention",     CMD_ATTENTION },
   { "author",        CMD_AUTHOR },
+  { "authors",       CMD_AUTHORS },
   { "b",             CMD_BOLD },
   { "c",             CMD_CODE },
   { "code",          CMD_STARTCODE },
   { "copydoc",       CMD_COPYDOC },
+  { "copybrief",     CMD_COPYBRIEF },
+  { "copydetails",   CMD_COPYDETAILS },
   { "date",          CMD_DATE },
   { "dontinclude",   CMD_DONTINCLUDE },
   { "dotfile",       CMD_DOTFILE },
@@ -89,9 +92,8 @@ CommandMap cmdMap[] =
   { "verbinclude",   CMD_VERBINCLUDE },
   { "version",       CMD_VERSION },
   { "warning",       CMD_WARNING },
-  { "author",        CMD_AUTHOR },
-  { "authors",       CMD_AUTHORS },
   { "throws",        CMD_EXCEPTION },
+  { "tparam",        CMD_TPARAM },
   { "\\",            CMD_BSLASH },
   { "@",             CMD_AT },
   { "<",             CMD_LESS },
@@ -100,6 +102,7 @@ CommandMap cmdMap[] =
   { "$",             CMD_DOLLAR },
   { "#",             CMD_HASH },
   { "%",             CMD_PERCENT },
+  { "\"",            CMD_QUOTE },
   { "_internalref",  CMD_INTERNALREF },
   { "dot",           CMD_DOT },
   { "msc",           CMD_MSC },
@@ -109,7 +112,8 @@ CommandMap cmdMap[] =
   { "endmanonly",    CMD_ENDMANONLY },
   { "includelineno", CMD_INCWITHLINES },
   { "inheritdoc",    CMD_INHERITDOC },
-  { 0,               0 }
+  { "mscfile",       CMD_MSCFILE },
+  { 0,               0 },
 };
 
 //----------------------------------------------------------------------------
@@ -155,31 +159,35 @@ CommandMap htmlTagMap[] =
   { "span",       HTML_SPAN },
   { "div",        HTML_DIV },
 
-  { "c",          XML_C },
-  // { "code",       XML_CODE },  <= ambigious <code> is also a HTML tag
-  { "description",XML_DESCRIPTION },
-  { "example",    XML_EXAMPLE },
-  { "exception",  XML_EXCEPTION },
-  { "include",    XML_INCLUDE },
-  { "item",       XML_ITEM },
-  { "list",       XML_LIST },
-  { "para",       XML_PARA },
-  { "param",      XML_PARAM },
-  { "paramref",   XML_PARAMREF },
-  { "permission", XML_PERMISSION },
-  { "remarks",    XML_REMARKS },
-  { "returns",    XML_RETURNS },
-  { "see",        XML_SEE },
-  { "seealso",    XML_SEEALSO },
-  { "summary",    XML_SUMMARY },
-  { "value",      XML_VALUE },
-  { 0,            0 }
+  { "c",            XML_C },
+  // { "code",       XML_CODE },  <= ambiguous <code> is also a HTML tag
+  { "description",  XML_DESCRIPTION },
+  { "example",      XML_EXAMPLE },
+  { "exception",    XML_EXCEPTION },
+  { "include",      XML_INCLUDE },
+  { "item",         XML_ITEM },
+  { "list",         XML_LIST }, // type="table|bullet|number"
+  { "listheader",   XML_LISTHEADER },
+  { "para",         XML_PARA },
+  { "param",        XML_PARAM },
+  { "paramref",     XML_PARAMREF },
+  { "typeparam",    XML_TYPEPARAM },
+  { "typeparamref", XML_TYPEPARAMREF },
+  { "permission",   XML_PERMISSION },
+  { "remarks",      XML_REMARKS },
+  { "returns",      XML_RETURNS },
+  { "see",          XML_SEE },
+  { "seealso",      XML_SEEALSO },
+  { "summary",      XML_SUMMARY },
+  { "term",         XML_TERM },
+  { "value",        XML_VALUE },
+  { 0,              0 }
 };
 
 //----------------------------------------------------------------------------
 
-Mapper *Mappers::cmdMapper     = new Mapper(cmdMap);
-Mapper *Mappers::htmlTagMapper = new Mapper(htmlTagMap);
+Mapper *Mappers::cmdMapper     = new Mapper(cmdMap,TRUE);
+Mapper *Mappers::htmlTagMapper = new Mapper(htmlTagMap,FALSE);
 
 void Mappers::freeMappers()
 {
