@@ -113,7 +113,8 @@ enum CommandType
   CMD_TPARAM       = 79 | SIMPLESECT_BIT,
   CMD_COPYBRIEF    = 80,
   CMD_COPYDETAILS  = 81,
-  CMD_QUOTE        = 82 
+  CMD_QUOTE        = 82,
+  CMD_MSCFILE      = 83 
 };
 
 enum HtmlTagType
