@@ -90,6 +90,7 @@
 "    <xsd:simpleContent>\n"
 "      <xsd:extension base=\"xsd:string\">\n"
 "        <xsd:attribute name=\"refid\" type=\"xsd:string\" />\n"
+"        <xsd:attribute name=\"prot\" type=\"DoxProtectionKind\" use=\"optional\"/>\n"
 "      </xsd:extension>\n"
 "    </xsd:simpleContent>\n"
 "  </xsd:complexType>\n"
@@ -99,7 +100,8 @@
 "      <xsd:extension base=\"xsd:string\">\n"
 "       <xsd:attribute name=\"refid\" type=\"xsd:string\" />\n"
 "       <xsd:attribute name=\"kindref\" type=\"DoxRefKind\" />\n"
-"       <xsd:attribute name=\"external\" type=\"xsd:string\" />\n"
+"       <xsd:attribute name=\"external\" type=\"xsd:string\" use=\"optional\"/>\n"
+"       <xsd:attribute name=\"tooltip\" type=\"xsd:string\" use=\"optional\"/>\n"
 "      </xsd:extension>\n"
 "    </xsd:simpleContent>\n"
 "  </xsd:complexType>\n"
@@ -122,6 +124,7 @@
 "      <xsd:element name=\"name\" />\n"
 "      <xsd:element name=\"read\" minOccurs=\"0\" />\n"
 "      <xsd:element name=\"write\" minOccurs=\"0\" />\n"
+"      <xsd:element name=\"bitfield\" minOccurs=\"0\" />\n"
 "      <xsd:element name=\"reimplements\" type=\"reimplementType\" minOccurs=\"0\" maxOccurs=\"unbounded\" />\n"
 "      <xsd:element name=\"reimplementedby\" type=\"reimplementType\" minOccurs=\"0\" maxOccurs=\"unbounded\" />\n"
 "      <xsd:element name=\"param\" type=\"paramType\" minOccurs=\"0\" maxOccurs=\"unbounded\" />\n"
@@ -145,8 +148,27 @@
 "    <xsd:attribute name=\"virt\" type=\"DoxVirtualKind\" />\n"
 "    <xsd:attribute name=\"volatile\" type=\"DoxBool\" />\n"
 "    <xsd:attribute name=\"mutable\" type=\"DoxBool\" />\n"
+"    <!-- Qt property -->\n"
 "    <xsd:attribute name=\"readable\" type=\"DoxBool\" use=\"optional\"/>\n"
 "    <xsd:attribute name=\"writable\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <!-- C++/CLI variable -->\n"
+"    <xsd:attribute name=\"initonly\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <!-- C++/CLI and C# property -->\n"
+"    <xsd:attribute name=\"settable\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <xsd:attribute name=\"gettable\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <!-- C++/CLI function -->\n"
+"    <xsd:attribute name=\"final\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <xsd:attribute name=\"sealed\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <xsd:attribute name=\"new\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <!-- C++/CLI event -->\n"
+"    <xsd:attribute name=\"add\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <xsd:attribute name=\"remove\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <xsd:attribute name=\"raise\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <!-- Objective-C 2.0 protocol method -->\n"
+"    <xsd:attribute name=\"optional\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <xsd:attribute name=\"required\" type=\"DoxBool\" use=\"optional\"/>\n"
+"    <!-- Objective-C 2.0 property accessor -->\n"
+"    <xsd:attribute name=\"accessor\" type=\"DoxAccessor\" use=\"optional\"/>\n"
 "  </xsd:complexType>\n"
 "\n"
 "  <xsd:complexType name=\"descriptionType\" mixed=\"true\">\n"
@@ -443,7 +465,10 @@
 "  <xsd:complexType name=\"docSimpleSectType\">\n"
 "    <xsd:sequence>\n"
 "      <xsd:element name=\"title\" type=\"docTitleType\" minOccurs=\"0\" />\n"
-"      <xsd:element name=\"para\" type=\"docParaType\" minOccurs=\"0\" maxOccurs=\"unbounded\" />\n"
+"      <xsd:sequence minOccurs=\"0\" maxOccurs=\"unbounded\">\n"
+"        <xsd:element name=\"para\" type=\"docParaType\" minOccurs=\"1\" maxOccurs=\"unbounded\" />\n"
+"        <xsd:element name=\"simplesectsep\" type=\"docEmptyType\" minOccurs=\"0\"/>\n"
+"      </xsd:sequence>\n"
 "    </xsd:sequence>\n"
 "    <xsd:attribute name=\"kind\" type=\"DoxSimpleSectKind\" />\n"
 "  </xsd:complexType>\n"
@@ -552,6 +577,7 @@
 "\n"
 "  <xsd:complexType name=\"docParamNameList\">\n"
 "    <xsd:sequence>\n"
+"      <xsd:element name=\"parametertype\" type=\"docParamType\" minOccurs=\"0\" maxOccurs=\"unbounded\" />\n"
 "      <xsd:element name=\"parametername\" type=\"docParamName\" minOccurs=\"0\" maxOccurs=\"unbounded\" />\n"
 "    </xsd:sequence>\n"
 "  </xsd:complexType>\n"
@@ -560,6 +586,12 @@
 "    <xsd:sequence>\n"
 "      <xsd:element name=\"ref\" type=\"refTextType\" minOccurs=\"0\" maxOccurs=\"1\" />\n"
 "    </xsd:sequence>\n"
+"  </xsd:complexType>\n"
+"\n"
+"  <xsd:complexType name=\"docParamName\" mixed=\"true\">\n"
+"    <xsd:sequence>\n"
+"      <xsd:element name=\"ref\" type=\"refTextType\" minOccurs=\"0\" maxOccurs=\"1\" />\n"
+"    </xsd:sequence>\n"
 "    <xsd:attribute name=\"direction\" type=\"DoxParamDir\" use=\"optional\" />\n"
 "  </xsd:complexType>\n"
 "\n"
@@ -759,6 +791,7 @@
 "      <xsd:enumeration value=\"param\" />\n"
 "      <xsd:enumeration value=\"retval\" />\n"
 "      <xsd:enumeration value=\"exception\" />\n"
+"      <xsd:enumeration value=\"templateparam\" />\n"
 "    </xsd:restriction>\n"
 "  </xsd:simpleType>\n"
 "\n"
@@ -776,5 +809,13 @@
 "    </xsd:restriction>\n"
 "  </xsd:simpleType>\n"
 "\n"
+"  <xsd:simpleType name=\"DoxAccessor\">\n"
+"    <xsd:restriction base=\"xsd:string\">\n"
+"      <xsd:enumeration value=\"retain\"/>\n"
+"      <xsd:enumeration value=\"copy\"/>\n"
+"      <xsd:enumeration value=\"assign\"/>\n"
+"    </xsd:restriction>\n"
+"  </xsd:simpleType>\n"
+"\n"
 "</xsd:schema>\n"
 "\n"
