@@ -8,6 +8,8 @@
 #include "language.h"
 #include "message.h"
 #include "dot.h"
+#include "layout.h"
+#include "ftextstream.h"
 
 //----------------------------------------------------------------------
 // method implementation
@@ -107,25 +109,37 @@ QCString DirDef::getOutputFileBase() const
   //return QCString().sprintf("dir_%06d",m_dirCount);
 }
 
-void DirDef::writeDetailedDocumentation(OutputList &ol)
+void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
-  if (!briefDescription().isEmpty() || !documentation().isEmpty())
+  if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) || 
+      !documentation().isEmpty())
   {
     ol.writeRuler();
     ol.pushGeneratorState();
-      ol.disable(OutputGenerator::Latex);
-      ol.disable(OutputGenerator::RTF);
-    ol.writeAnchor(0,"_details");
+    ol.disableAllBut(OutputGenerator::Html);
+      ol.writeAnchor(0,"_details");
     ol.popGeneratorState();
     ol.startGroupHeader();
-    ol.parseText(theTranslator->trDetailedDescription());
+    ol.parseText(title);
     ol.endGroupHeader();
 
     // repeat brief description
     if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
     {
       ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
-      ol.newParagraph();
+    }
+    // separator between brief and details
+    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") && 
+        !documentation().isEmpty())
+    {
+      ol.pushGeneratorState();
+        ol.disable(OutputGenerator::Man);
+        ol.disable(OutputGenerator::RTF);
+        // ol.newParagraph();  // FIXME:PARA
+        ol.enableAll();
+        ol.disableAllBut(OutputGenerator::Man);
+        ol.writeString("\n\n");
+      ol.popGeneratorState();
     }
 
     // write documentation
@@ -136,79 +150,64 @@ void DirDef::writeDetailedDocumentation(OutputList &ol)
   }
 }
 
-void DirDef::writeDocumentation(OutputList &ol)
+void DirDef::writeBriefDescription(OutputList &ol)
 {
-  ol.pushGeneratorState();
-  
-  QCString shortTitle=theTranslator->trDirReference(m_shortName);
-  QCString title=theTranslator->trDirReference(m_dispName);
-  startFile(ol,getOutputFileBase(),name(),title);
-
-  // write navigation path
-  writeNavigationPath(ol);
-
-  startTitle(ol,getOutputFileBase());
-  ol.pushGeneratorState();
-    ol.disableAllBut(OutputGenerator::Html);
-    ol.parseText(shortTitle);
-    ol.enableAll();
-    ol.disable(OutputGenerator::Html);
-    ol.parseText(title);
-  ol.popGeneratorState();
-  endTitle(ol,getOutputFileBase(),title);
-
-  // write brief or details (if DETAILS_AT_TOP)
-  if (Config_getBool("DETAILS_AT_TOP"))
-  {
-    writeDetailedDocumentation(ol);
-    ol.newParagraph();
-  }
-  else if (!briefDescription().isEmpty())
+  if (!briefDescription().isEmpty()) 
   {
+    ol.startParagraph();
     ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
-    ol.writeString(" \n");
     ol.pushGeneratorState();
-    ol.disable(OutputGenerator::Latex);
     ol.disable(OutputGenerator::RTF);
-    ol.disable(OutputGenerator::Man);
-    ol.startTextLink(0,"_details");
-    ol.parseText(theTranslator->trMore());
-    ol.endTextLink();
-    ol.enableAll();
-    ol.disableAllBut(OutputGenerator::Man);
-    ol.newParagraph();
-    ol.popGeneratorState();
-  }
+    ol.writeString(" \n");
+    ol.enable(OutputGenerator::RTF);
 
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    Doxygen::tagFile << "  <compound kind=\"dir\">" << endl;
-    Doxygen::tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
-    Doxygen::tagFile << "    <path>" << convertToXML(name()) << "</path>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
+    if (Config_getBool("REPEAT_BRIEF") ||
+        !documentation().isEmpty()
+       )
+    {
+      ol.disableAllBut(OutputGenerator::Html);
+      ol.startTextLink(0,"_details");
+      ol.parseText(theTranslator->trMore());
+      ol.endTextLink();
+    }
+    ol.popGeneratorState();
+
+    //ol.pushGeneratorState();
+    //ol.disable(OutputGenerator::RTF);
+    //ol.newParagraph();
+    //ol.popGeneratorState();
+    ol.endParagraph();
   }
-  
+  ol.writeSynopsis();
+}
+
+void DirDef::writeDirectoryGraph(OutputList &ol)
+{
   // write graph dependency graph
-  if (Config_getBool("DIRECTORY_GRAPH") && Config_getBool("HAVE_DOT"))
+  if (/*Config_getBool("DIRECTORY_GRAPH") &&*/ Config_getBool("HAVE_DOT"))
   {
     DotDirDeps dirDep(this);
     if (!dirDep.isTrivial())
     {
       msg("Generating dependency graph for directory %s\n",displayName().data());
       ol.disable(OutputGenerator::Man);
-      ol.newParagraph();
+      ol.startParagraph();
       ol.startDirDepGraph();
       //TODO: ol.parseText(theTranslator->trDirDepGraph());
+      ol.parseText((QCString)"Directory dependency graph for "+displayName()+":");
       ol.endDirDepGraph(dirDep);
+      ol.endParagraph();
       ol.enableAll();
     }
   }
+}
 
-  ol.startMemberSections();
+void DirDef::writeSubDirList(OutputList &ol)
+{
   // write subdir list
   if (m_subdirs.count()>0)
   {
-    ol.startMemberHeader();
+    ol.startMemberHeader("subdirs");
     ol.parseText(theTranslator->trDir(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
@@ -226,6 +225,7 @@ void DirDef::writeDocumentation(OutputList &ol)
       }
       if (!dd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
+        ol.startParagraph();
         ol.startMemberDescription();
         ol.parseDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),
             FALSE, // indexWords
@@ -235,18 +235,21 @@ void DirDef::writeDocumentation(OutputList &ol)
             TRUE   // link from index
            );
         ol.endMemberDescription();
-        ol.newParagraph();
+        ol.endParagraph();
       }
       dd=m_subdirs.next();
     }
 
     ol.endMemberList();
   }
-  
+}
+
+void DirDef::writeFileList(OutputList &ol)
+{
   // write file list
   if (m_fileList->count()>0)
   {
-    ol.startMemberHeader();
+    ol.startMemberHeader("files");
     ol.parseText(theTranslator->trFile(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
@@ -285,6 +288,7 @@ void DirDef::writeDocumentation(OutputList &ol)
       ol.endMemberItem();
       if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
+        ol.startParagraph();
         ol.startMemberDescription();
         ol.parseDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),
             FALSE, // indexWords
@@ -294,13 +298,123 @@ void DirDef::writeDocumentation(OutputList &ol)
             TRUE   // link from index
            );
         ol.endMemberDescription();
-        ol.newParagraph();
+        ol.endParagraph();
       }
       fd=m_fileList->next();
     }
     ol.endMemberList();
   }
+}
+
+void DirDef::startMemberDeclarations(OutputList &ol)
+{
+  ol.startMemberSections();
+}
+
+void DirDef::endMemberDeclarations(OutputList &ol)
+{
   ol.endMemberSections();
+}
+
+void DirDef::writeDocumentation(OutputList &ol)
+{
+  ol.pushGeneratorState();
+  
+  QCString shortTitle=theTranslator->trDirReference(m_shortName);
+  QCString title=theTranslator->trDirReference(m_dispName);
+  startFile(ol,getOutputFileBase(),name(),title,HLI_None,TRUE);
+
+  // write navigation path
+  writeNavigationPath(ol);
+
+  ol.endQuickIndices();
+
+  startTitle(ol,getOutputFileBase());
+  ol.pushGeneratorState();
+    ol.disableAllBut(OutputGenerator::Html);
+    ol.parseText(shortTitle);
+    ol.enableAll();
+    ol.disable(OutputGenerator::Html);
+    ol.parseText(title);
+  ol.popGeneratorState();
+  endTitle(ol,getOutputFileBase(),title);
+  ol.startContents();
+
+  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
+  {
+    Doxygen::tagFile << "  <compound kind=\"dir\">" << endl;
+    Doxygen::tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
+    Doxygen::tagFile << "    <path>" << convertToXML(name()) << "</path>" << endl;
+    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
+  }
+  
+  //---------------------------------------- start flexible part -------------------------------
+
+  QListIterator<LayoutDocEntry> eli(
+      LayoutDocManager::instance().docEntries(LayoutDocManager::Directory));
+  LayoutDocEntry *lde;
+  for (eli.toFirst();(lde=eli.current());++eli)
+  {
+    switch (lde->kind())
+    {
+      case LayoutDocEntry::BriefDesc: 
+        writeBriefDescription(ol);
+        break; 
+      case LayoutDocEntry::DirGraph: 
+        writeDirectoryGraph(ol);
+        break; 
+      case LayoutDocEntry::MemberDeclStart: 
+        startMemberDeclarations(ol);
+        break; 
+      case LayoutDocEntry::DirSubDirs: 
+        writeSubDirList(ol);
+        break; 
+      case LayoutDocEntry::DirFiles: 
+        writeFileList(ol);
+        break; 
+      case LayoutDocEntry::MemberDeclEnd: 
+        endMemberDeclarations(ol);
+        break;
+      case LayoutDocEntry::DetailedDesc: 
+        {
+          LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
+          writeDetailedDescription(ol,ls->title);
+        }
+        break;
+      case LayoutDocEntry::ClassIncludes:
+      case LayoutDocEntry::ClassInheritanceGraph:
+      case LayoutDocEntry::ClassNestedClasses:
+      case LayoutDocEntry::ClassCollaborationGraph:
+      case LayoutDocEntry::ClassAllMembersLink:
+      case LayoutDocEntry::ClassUsedFiles:
+      case LayoutDocEntry::NamespaceNestedNamespaces:
+      case LayoutDocEntry::NamespaceClasses:
+      case LayoutDocEntry::FileClasses:
+      case LayoutDocEntry::FileNamespaces:
+      case LayoutDocEntry::FileIncludes:
+      case LayoutDocEntry::FileIncludeGraph:
+      case LayoutDocEntry::FileIncludedByGraph: 
+      case LayoutDocEntry::FileSourceLink:
+      case LayoutDocEntry::GroupClasses: 
+      case LayoutDocEntry::GroupNamespaces:
+      case LayoutDocEntry::GroupDirs: 
+      case LayoutDocEntry::GroupNestedGroups: 
+      case LayoutDocEntry::GroupFiles:
+      case LayoutDocEntry::GroupGraph: 
+      case LayoutDocEntry::GroupPageDocs:
+      case LayoutDocEntry::AuthorSection:
+      case LayoutDocEntry::MemberGroups:
+      case LayoutDocEntry::MemberDecl:
+      case LayoutDocEntry::MemberDef:
+      case LayoutDocEntry::MemberDefStart:
+      case LayoutDocEntry::MemberDefEnd:
+        err("Internal inconsistency: member %d should not be part of "
+            "LayoutDocManager::Directory entry list\n",lde->kind());
+        break;
+    }
+  }
+
+  //---------------------------------------- end flexible part -------------------------------
 
   if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
   {
@@ -308,14 +422,10 @@ void DirDef::writeDocumentation(OutputList &ol)
     Doxygen::tagFile << "  </compound>" << endl;
   }
 
-
-  if (!Config_getBool("DETAILS_AT_TOP"))
-  {
-    writeDetailedDocumentation(ol);
-  }
-  
   endFile(ol); 
   ol.popGeneratorState();
+
+
 }
 
 #if 0
@@ -324,7 +434,7 @@ void DirDef::writePathFragment(OutputList &ol) const
   if (m_parent)
   {
     m_parent->writePathFragment(ol);
-    ol.writeString("&nbsp;/&nbsp;");
+    ol.writeString("&#160;/&#160;");
   }
   ol.writeObjectLink(getReference(),getOutputFileBase(),0,shortName());
 }
@@ -507,13 +617,13 @@ FilePair *UsedDir::findFilePair(const char *name)
 DirDef *DirDef::createNewDir(const char *path)
 {
   ASSERT(path!=0);
-  DirDef *dir = Doxygen::directories.find(path);
+  DirDef *dir = Doxygen::directories->find(path);
   if (dir==0) // new dir
   {
     //printf("Adding new dir %s\n",path);
     dir = new DirDef(path);
     //printf("createNewDir %s short=%s\n",path,dir->shortName().data());
-    Doxygen::directories.inSort(path,dir);
+    Doxygen::directories->inSort(path,dir);
   }
   return dir;
 }
@@ -553,147 +663,9 @@ DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
   return dir;
 }
 
-void DirDef::writeDepGraph(QTextStream &t)
+void DirDef::writeDepGraph(FTextStream &t)
 {
-    t << "digraph G {\n";
-    if (Config_getBool("DOT_TRANSPARENT"))
-    {
-      t << "  bgcolor=transparent;\n";
-    }
-    t << "  compound=true\n";
-    t << "  node [ fontsize=10, fontname=\"FreeSans.ttf\"];\n";
-    t << "  edge [ labelfontsize=9, labelfontname=\"FreeSans.ttf\"];\n";
-
-    QDict<DirDef> dirsInGraph(257);
-    
-    dirsInGraph.insert(getOutputFileBase(),this);
-    if (parent())
-    {
-      t << "  subgraph cluster" << parent()->getOutputFileBase() << " {\n";
-      t << "    graph [ bgcolor=\"#ddddee\", pencolor=\"black\", label=\"" 
-        << parent()->shortName() 
-        << "\" fontname=\"FreeSans.ttf\", fontsize=10, URL=\"";
-      t << parent()->getOutputFileBase() << Doxygen::htmlFileExtension;
-      t << "\"]\n";
-    }
-    if (isCluster())
-    {
-      t << "  subgraph cluster" << getOutputFileBase() << " {\n";
-      t << "    graph [ bgcolor=\"#eeeeff\", pencolor=\"black\", label=\"\""
-        << " URL=\"" << getOutputFileBase() << Doxygen::htmlFileExtension 
-        << "\"];\n";
-      t << "    " << getOutputFileBase() << " [shape=plaintext label=\"" 
-        << shortName() << "\"];\n";
-
-      // add nodes for sub directories
-      QListIterator<DirDef> sdi(m_subdirs);
-      DirDef *sdir;
-      for (sdi.toFirst();(sdir=sdi.current());++sdi)
-      {
-        t << "    " << sdir->getOutputFileBase() << " [shape=box label=\"" 
-          << sdir->shortName() << "\"";
-        if (sdir->isCluster())
-        {
-          t << " color=\"red\"";
-        }
-        else
-        {
-          t << " color=\"black\"";
-        }
-        t << " fillcolor=\"white\" style=\"filled\"";
-        t << " URL=\"" << sdir->getOutputFileBase() 
-          << Doxygen::htmlFileExtension << "\"";
-        t << "];\n";
-        dirsInGraph.insert(sdir->getOutputFileBase(),sdir);
-      }
-      t << "  }\n";
-    }
-    else
-    {
-      t << "  " << getOutputFileBase() << " [shape=box, label=\"" 
-        << shortName() << "\", style=\"filled\", fillcolor=\"#eeeeff\","
-        << " pencolor=\"black\", URL=\"" << getOutputFileBase() 
-        << Doxygen::htmlFileExtension << "\"];\n";
-    }
-    if (parent())
-    {
-      t << "  }\n";
-    }
-
-    // add nodes for other used directories
-    QDictIterator<UsedDir> udi(*m_usedDirs);
-    UsedDir *udir;
-    //printf("*** For dir %s\n",shortName().data());
-    for (udi.toFirst();(udir=udi.current());++udi) 
-      // for each used dir (=directly used or a parent of a directly used dir)
-    {
-      const DirDef *usedDir=udir->dir();
-      DirDef *dir=this;
-      while (dir)
-      {
-        //printf("*** check relation %s->%s same_parent=%d !%s->isParentOf(%s)=%d\n",
-        //    dir->shortName().data(),usedDir->shortName().data(),
-        //    dir->parent()==usedDir->parent(),
-        //    usedDir->shortName().data(),
-        //    shortName().data(),
-        //    !usedDir->isParentOf(this)
-        //    );
-        if (dir!=usedDir && dir->parent()==usedDir->parent() && !usedDir->isParentOf(this))
-          // include if both have the same parent (or no parent)
-        {
-          t << "  " << usedDir->getOutputFileBase() << " [shape=box label=\"" 
-            << usedDir->shortName() << "\"";
-          if (usedDir->isCluster())
-          {
-            if (!Config_getBool("DOT_TRANSPARENT"))
-            {
-              t << " fillcolor=\"white\" style=\"filled\"";
-            }
-            t << " color=\"red\"";
-          }
-          t << " URL=\"" << usedDir->getOutputFileBase() 
-            << Doxygen::htmlFileExtension << "\"];\n";
-          dirsInGraph.insert(usedDir->getOutputFileBase(),usedDir);
-          break;
-        }
-        dir=dir->parent();
-      }
-    }
-
-    // add relations between all selected directories
-    DirDef *dir;
-    QDictIterator<DirDef> di(dirsInGraph);
-    for (di.toFirst();(dir=di.current());++di) // foreach dir in the graph
-    {
-      QDictIterator<UsedDir> udi(*dir->usedDirs());
-      UsedDir *udir;
-      for (udi.toFirst();(udir=udi.current());++udi) // foreach used dir
-      {
-        const DirDef *usedDir=udir->dir();
-        if ((dir!=this || !udir->inherited()) &&     // only show direct dependendies for this dir
-            (usedDir!=this || !udir->inherited()) && // only show direct dependendies for this dir
-            !usedDir->isParentOf(dir) &&             // don't point to own parent
-            dirsInGraph.find(usedDir->getOutputFileBase())) // only point to nodes that are in the graph
-        {
-          QCString relationName;
-          relationName.sprintf("dir_%06d_%06d",dir->m_dirCount,usedDir->m_dirCount);
-          if (Doxygen::dirRelations.find(relationName)==0)
-          {
-            // new relation
-            Doxygen::dirRelations.append(relationName,
-                new DirRelation(relationName,dir,udir));
-          }
-          int nrefs = udir->filePairs().count();
-          t << "  " << dir->getOutputFileBase() << "->" 
-                    << usedDir->getOutputFileBase();
-          t << " [headlabel=\"" << nrefs << "\", labeldistance=1.5";
-          t << " headhref=\"" << relationName << Doxygen::htmlFileExtension 
-            << "\"];\n";
-        }
-      }
-    }
-
-    t << "}\n";
+    writeDotDirDepGraph(t,this);
 }
 
 //----------------------------------------------------------------------
@@ -703,7 +675,7 @@ static void writePartialDirPath(OutputList &ol,const DirDef *root,const DirDef *
   if (target->parent()!=root) 
   {
     writePartialDirPath(ol,root,target->parent());
-    ol.writeString("&nbsp;/&nbsp;");
+    ol.writeString("&#160;/&#160;");
   }
   ol.writeObjectLink(target->getReference(),target->getOutputFileBase(),0,target->shortName());
 }
@@ -713,7 +685,7 @@ static void writePartialFilePath(OutputList &ol,const DirDef *root,const FileDef
   if (fd->getDirDef() && fd->getDirDef()!=root)
   {
     writePartialDirPath(ol,root,fd->getDirDef());
-    ol.writeString("&nbsp;/&nbsp;");
+    ol.writeString("&#160;/&#160;");
   }
   if (fd->isLinkable())
   {
@@ -732,14 +704,17 @@ void DirRelation::writeDocumentation(OutputList &ol)
   ol.pushGeneratorState();
   ol.disableAllBut(OutputGenerator::Html);
 
-  QCString shortTitle=m_src->shortName()+" &rarr; "+
-                      m_dst->dir()->shortName()+" Relation";//theTranslator->trDirRelation(m_shortName);
-  QCString title=m_src->displayName()+" -> "+
-                 m_dst->dir()->shortName()+" Relation";//theTranslator->trDirRelation(m_dispName);
+  QCString shortTitle=theTranslator->trDirRelation(
+                      m_src->shortName()+" &rarr; "+
+                      m_dst->dir()->shortName());
+  QCString title=theTranslator->trDirRelation(
+                 m_src->displayName()+" -> "+
+                 m_dst->dir()->shortName());
   startFile(ol,getOutputFileBase(),getOutputFileBase(),title);
 
   // write navigation path
   m_src->writeNavigationPath(ol);
+  ol.startContents();
 
   //startTitle(ol,getOutputFileBase());
   //  ol.parseText(shortTitle);
@@ -748,11 +723,16 @@ void DirRelation::writeDocumentation(OutputList &ol)
   
   ol.writeString("<table class=\"dirtab\">");
   ol.writeString("<tr class=\"dirtab\">");
-  ol.writeString("<th class=\"dirtab\">File in ");
-  m_src->writePathFragment(ol);
+  // TODO: translate me! "File in %s"
+  ol.writeString("<th class=\"dirtab\">");
+  ol.parseText(theTranslator->trFileIn(m_src->pathFragment()));
+  //m_src->writePathFragment(ol);
   ol.writeString("</th>");
-  ol.writeString("<th class=\"dirtab\">Includes file in ");
-  m_dst->dir()->writePathFragment(ol);
+  // TODO: translate me! "Includes file in %s"
+  ol.writeString("<th class=\"dirtab\">");
+  //ol.writeString("Includes file in ");
+  //m_dst->dir()->writePathFragment(ol);
+  ol.parseText(theTranslator->trIncludesFileIn(m_dst->dir()->pathFragment()));
   ol.writeString("</th>");
   ol.writeString("</tr>");
 
@@ -781,7 +761,7 @@ void DirRelation::writeDocumentation(OutputList &ol)
 void buildDirectories()
 {
   // for each input file
-  FileNameListIterator fnli(Doxygen::inputNameList); 
+  FileNameListIterator fnli(*Doxygen::inputNameList); 
   FileName *fn;
   for (fnli.toFirst();(fn=fnli.current());++fnli)
   {
@@ -793,7 +773,7 @@ void buildDirectories()
       if (fd->getReference().isEmpty() && !fd->isDocumentationFile())
       {
         DirDef *dir;
-        if ((dir=Doxygen::directories.find(fd->getPath()))==0) // new directory
+        if ((dir=Doxygen::directories->find(fd->getPath()))==0) // new directory
         {
           dir = DirDef::mergeDirectoryInTree(fd->getPath());
         }
@@ -809,7 +789,7 @@ void buildDirectories()
   //DirDef *root = new DirDef("root:");
   // compute relations between directories => introduce container dirs.
   DirDef *dir;
-  DirSDict::Iterator sdi(Doxygen::directories);
+  DirSDict::Iterator sdi(*Doxygen::directories);
   for (sdi.toFirst();(dir=sdi.current());++sdi)
   {
     //printf("New dir %s\n",dir->displayName().data());
@@ -817,7 +797,7 @@ void buildDirectories()
     int i=name.findRev('/',name.length()-2);
     if (i>0)
     {
-      DirDef *parent = Doxygen::directories.find(name.left(i+1));
+      DirDef *parent = Doxygen::directories->find(name.left(i+1));
       //if (parent==0) parent=root;
       if (parent) 
       {
@@ -832,7 +812,7 @@ void buildDirectories()
 void computeDirDependencies()
 {
   DirDef *dir;
-  DirSDict::Iterator sdi(Doxygen::directories);
+  DirSDict::Iterator sdi(*Doxygen::directories);
   // compute nesting level for each directory
   for (sdi.toFirst();(dir=sdi.current());++sdi)
   {
@@ -872,12 +852,12 @@ void computeDirDependencies()
 #endif
 }
 
-
+#if 0
 void writeDirDependencyGraph(const char *dirName)
 {
   QString path;
   DirDef *dir;
-  DirSDict::Iterator sdi(Doxygen::directories);
+  DirSDict::Iterator sdi(*Doxygen::directories);
   QFile htmlPage(QCString(dirName)+"/dirdeps.html");
   if (htmlPage.open(IO_WriteOnly))
   {
@@ -908,7 +888,7 @@ void writeDirDependencyGraph(const char *dirName)
       
       //QCString dotArgs(4096);
       //dotArgs.sprintf("%s -Tgif -o %s",path.data(),outFile.data());
-      //if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs,FALSE)!=0)
+      //if (portable_system(Config_getString("DOT_PATH")+"dot",dotArgs,FALSE)!=0)
       //{
       //  err("Problems running dot. Check your installation!\n");
       //}
@@ -917,11 +897,12 @@ void writeDirDependencyGraph(const char *dirName)
   }
   htmlPage.close();
 }
+#endif
 
 void generateDirDocs(OutputList &ol)
 {
   DirDef *dir;
-  DirSDict::Iterator sdi(Doxygen::directories);
+  DirSDict::Iterator sdi(*Doxygen::directories);
   for (sdi.toFirst();(dir=sdi.current());++sdi)
   {
     dir->writeDocumentation(ol);
