@@ -158,6 +158,7 @@
 "	border-radius: 0.5em;\n"
 "	-webkit-border-radius: .5em;\n"
 "	-moz-border-radius: .5em;\n"
+"	box-shadow: 2px 2px 3px #999;\n"
 "	-webkit-box-shadow: 2px 2px 3px #999;\n"
 "	-moz-box-shadow: rgba(0, 0, 0, 0.15) 2px 2px 2px;\n"
 "	background-image: -webkit-gradient(linear, left top, left bottom, from(#eee), to(#000),color-stop(0.3, #444));\n"
@@ -167,7 +168,6 @@
 "div.groupHeader {\n"
 "	margin-left: 16px;\n"
 "	margin-top: 12px;\n"
-"	margin-bottom: 6px;\n"
 "	font-weight: bold;\n"
 "}\n"
 "\n"
@@ -376,7 +376,7 @@
 "	font-size: 80%;\n"
 "	color: ##60;\n"
 "	font-weight: normal;\n"
-"	margin-left: 3px;\n"
+"	margin-left: 9px;\n"
 "}\n"
 "\n"
 ".memnav {\n"
@@ -407,6 +407,10 @@
 "        color: ##2b;\n"
 "        font-weight: bold;\n"
 "        text-shadow: 0px 1px 1px rgba(255, 255, 255, 0.9);\n"
+"        /* opera specific markup */\n"
+"        box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
+"        border-top-right-radius: 8px;\n"
+"        border-top-left-radius: 8px;\n"
 "        /* firefox specific markup */\n"
 "        -moz-box-shadow: rgba(0, 0, 0, 0.15) 5px 5px 5px;\n"
 "        -moz-border-radius-topright: 8px;\n"
@@ -428,6 +432,10 @@
 "        padding: 2px 5px;\n"
 "        background-color: ##FC;\n"
 "        border-top-width: 0;\n"
+"        /* opera specific markup */\n"
+"        border-bottom-left-radius: 8px;\n"
+"        border-bottom-right-radius: 8px;\n"
+"        box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.15);\n"
 "        /* firefox specific markup */\n"
 "        -moz-border-radius-bottomleft: 8px;\n"
 "        -moz-border-radius-bottomright: 8px;\n"
@@ -456,6 +464,28 @@
 "	font-style: normal;\n"
 "}\n"
 "\n"
+".params, .retval, .exception, .tparams {\n"
+"        border-spacing: 6px 2px;\n"
+"}       \n"
+"\n"
+".params .paramname, .retval .paramname {\n"
+"        font-weight: bold;\n"
+"        vertical-align: top;\n"
+"}\n"
+"        \n"
+".params .paramtype {\n"
+"        font-style: italic;\n"
+"        vertical-align: top;\n"
+"}       \n"
+"        \n"
+".params .paramdir {\n"
+"        font-family: \"courier new\",courier,monospace;\n"
+"        vertical-align: top;\n"
+"}\n"
+"\n"
+"\n"
+"\n"
+"\n"
 "/* @end */\n"
 "\n"
 "/* @group Directory (tree) */\n"
