@@ -1,8 +1,19 @@
 /******************************************************************************
  * ftvhelp.cpp,v 1.0 2000/09/06 16:09:00
  *
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ *
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation under the terms of the GNU General Public License is hereby 
+ * granted. No representations are made about the suitability of this software 
+ * for any purpose. It is provided "as is" without express or implied warranty.
+ * See the GNU General Public License for more details.
+ *
+ * Documents produced by Doxygen are derivative works derived from the
+ * input used in their production; they are not affected by this license.
+ *
  * Contributed by Kenney Wong <kwong@ea.com>
- * Modified by Dimitri van Heesch (c) 2003
+ * Modified by Dimitri van Heesch
  *
  * Folder Tree View for offline help on browsers that do not support HTML Help.
  */
@@ -18,6 +29,7 @@
 #include "message.h"
 #include "doxygen.h"
 #include "language.h"
+#include "htmlgen.h"
 
 #define MAX_INDENT 1024
 
@@ -293,7 +305,7 @@ unsigned char ftv2vertline_png[] = {
 
 FTVImageInfo image_info[] =
 {
-  { "&nbsp;", "ftv2blank.png",ftv2blank_png,174,16,22 },
+  { "&#160;", "ftv2blank.png",ftv2blank_png,174,16,22 },
   { "*",  "ftv2doc.png",ftv2doc_png,255,24,22 },
   { "+",  "ftv2folderclosed.png",ftv2folderclosed_png,259,24,22 },
   { "-",  "ftv2folderopen.png",ftv2folderopen_png,261,24,22 },
@@ -326,20 +338,17 @@ struct FTVNode
 
 //----------------------------------------------------------------------------
 
-FTVHelp *FTVHelp::m_theInstance = 0;
-
-/*! Construm_cts an ftv help object. 
+/*! Constructs an ftv help object. 
  *  The object has to be \link initialize() initialized\endlink before it can 
  *  be used.
  */
-FTVHelp::FTVHelp() 
+FTVHelp::FTVHelp(bool TLI) 
 {
   /* initial depth */
-  //m_dc = 0;
-  //m_cf = 0;
   m_indentNodes = new QList<FTVNode>[MAX_INDENT];
   m_indentNodes[0].setAutoDelete(TRUE);
   m_indent=0;
+  m_topLevelIndex = TLI;
 }
 
 FTVHelp::~FTVHelp()
@@ -347,16 +356,6 @@ FTVHelp::~FTVHelp()
   delete[] m_indentNodes;
 }
 
-#if 0
-/*! return a reference to the one and only instance of this class. 
- */
-FTVHelp *FTVHelp::getInstance()
-{
-  if (m_theInstance==0) m_theInstance = new FTVHelp;
-  return m_theInstance;
-}
-#endif
-
 /*! This will create a folder tree view table of contents file (tree.js).
  *  \sa finalize()
  */
@@ -371,9 +370,6 @@ void FTVHelp::initialize()
 void FTVHelp::finalize()
 {
   generateTreeView();
-  generateTreeViewImages();
-  delete m_theInstance;
-  m_theInstance=0;
 }
 
 /*! Increase the level of the contents hierarchy. 
@@ -382,10 +378,8 @@ void FTVHelp::finalize()
  */
 void FTVHelp::incContentsDepth()
 {
-  //int i; for (i=0;i<m_dc+1;i++) m_cts << "  ";
   m_indent++;
   ASSERT(m_indent<MAX_INDENT);
-  //return m_indent;
 }
 
 /*! Decrease the level of the contents hierarchy.
@@ -394,8 +388,6 @@ void FTVHelp::incContentsDepth()
  */
 void FTVHelp::decContentsDepth()
 {
-  //int i; for (i=0;i<m_dc;i++) m_cts << "  ";
-
   ASSERT(m_indent>0);
   if (m_indent>0)
   {
@@ -408,7 +400,6 @@ void FTVHelp::decContentsDepth()
       parent->children.append(children->take(0));
     }
   }
-  //return m_indent;
 }
 
 /*! Add a list item to the contents file.
@@ -423,7 +414,7 @@ void FTVHelp::addContentsItem(bool isDir,
                               const char *ref,
                               const char *file,
                               const char *anchor
-                             )
+                              )
 {
   QList<FTVNode> *nl = &m_indentNodes[m_indent];
   FTVNode *newNode = new FTVNode(isDir,ref,file,anchor,name);
@@ -442,7 +433,7 @@ void FTVHelp::addContentsItem(bool isDir,
 
 static int folderId=1;
 
-void FTVHelp::generateIndent(QTextStream &t, FTVNode *n,int level)
+void FTVHelp::generateIndent(FTextStream &t, FTVNode *n,int level)
 {
   if (n->parent)
   {
@@ -487,46 +478,43 @@ void FTVHelp::generateIndent(QTextStream &t, FTVNode *n,int level)
   }
 }
 
-void FTVHelp::generateLink(QTextStream &t,FTVNode *n)
+void FTVHelp::generateLink(FTextStream &t,FTVNode *n)
 {
-  QCString *dest;
   //printf("FTVHelp::generateLink(ref=%s,file=%s,anchor=%s\n",
   //    n->ref.data(),n->file.data(),n->anchor.data());
   if (n->file.isEmpty()) // no link
   {
-    t << "<b>" << n->name << "</b>";
+    t << "<b>" << convertToHtml(n->name) << "</b>";
   }
   else // link into other frame
   {
     if (!n->ref.isEmpty()) // link to entity imported via tag file
     {
       t << "<a class=\"elRef\" ";
-      t << "doxygen=\"" << n->ref << ":";
-      if ((dest=Doxygen::tagDestinationDict[n->ref])) t << *dest << "/";
-      t << "\" ";
+      t << externalLinkTarget() << externalRef("",n->ref,FALSE);
     }
     else // local link
     {
       t << "<a class=\"el\" ";
     }
     t << "href=\"";
-    if (!n->ref.isEmpty())
-    {
-      if ((dest=Doxygen::tagDestinationDict[n->ref])) t << *dest << "/";
-    }
+    t << externalRef("",n->ref,TRUE);
     t << n->file << Doxygen::htmlFileExtension;
     if (!n->anchor.isEmpty()) t << "#" << n->anchor;
-    t << "\" target=\"basefrm\">";
-    t << n->name;
+    if (m_topLevelIndex)
+      t << "\" target=\"basefrm\">";
+    else
+      t << "\" target=\"_self\">";
+    t << convertToHtml(n->name);
     t << "</a>";
     if (!n->ref.isEmpty())
     {
-      t << "&nbsp;[external]";
+      t << "&#160;[external]";
     }
   }
 }
 
-void FTVHelp::generateTree(QTextStream &t, const QList<FTVNode> &nl,int level)
+void FTVHelp::generateTree(FTextStream &t, const QList<FTVNode> &nl,int level)
 {
   QCString spaces;
   spaces.fill(' ',level*2+8);
@@ -573,18 +561,98 @@ void FTVHelp::generateTreeViewImages()
     }
     else
     {
-      fprintf(stderr,"Warning: Cannot open file %s for writing\n",fileName.data());
+      fprintf(stderr,"warning: Cannot open file %s for writing\n",fileName.data());
     }
     f.close();
     p++;
   }
 } 
 
+void FTVHelp::generateScript(FTextStream &t)
+{
+  t << "    <script type=\"text/javascript\">\n";
+  t << "    <!-- // Hide script from old browsers\n";
+  t << "    \n";
+
+  /* User has clicked on a node (folder or +/-) in the tree */
+  t << "    function toggleFolder(id, imageNode) \n";
+  t << "    {\n";
+  t << "      var folder = document.getElementById(id);\n";
+  t << "      var l = imageNode.src.length;\n";
+  /* If the user clicks on the book icon, we move left one image so 
+   * the code (below) will also adjust the '+' icon. 
+   */
+  t << "      if (imageNode.src.substring(l-20,l)==\"" FTV_ICON_FILE(folderclosed) "\" || \n";
+  t << "          imageNode.src.substring(l-18,l)==\"" FTV_ICON_FILE(folderopen)  "\")\n";
+  t << "      {\n";
+  t << "        imageNode = imageNode.previousSibling;\n";
+  t << "        l = imageNode.src.length;\n";
+  t << "      }\n";
+  t << "      if (folder == null) \n";
+  t << "      {\n";
+  t << "      } \n";
+  /* Node controls a open section, we need to close it */
+  t << "      else if (folder.style.display == \"block\") \n";
+  t << "      {\n";
+  t << "        if (imageNode != null) \n";
+  t << "        {\n";
+  t << "          imageNode.nextSibling.src = \"" FTV_ICON_FILE(folderclosed) "\";\n";
+  t << "          if (imageNode.src.substring(l-13,l) == \"" FTV_ICON_FILE(mnode) "\")\n";
+  t << "          {\n";
+  t << "            imageNode.src = \"" FTV_ICON_FILE(pnode) "\";\n";
+  t << "          }\n";
+  t << "          else if (imageNode.src.substring(l-17,l) == \"" FTV_ICON_FILE(mlastnode) "\")\n";
+  t << "          {\n";
+  t << "            imageNode.src = \"" FTV_ICON_FILE(plastnode) "\";\n";
+  t << "          }\n";
+  t << "        }\n";
+  t << "        folder.style.display = \"none\";\n";
+  t << "      } \n";
+  t << "      else \n"; /* section is closed, we need to open it */
+  t << "      {\n";
+  t << "        if (imageNode != null) \n";
+  t << "        {\n";
+  t << "          imageNode.nextSibling.src = \"" FTV_ICON_FILE(folderopen) "\";\n";
+  t << "          if (imageNode.src.substring(l-13,l) == \"" FTV_ICON_FILE(pnode) "\")\n";
+  t << "          {\n";
+  t << "            imageNode.src = \"" FTV_ICON_FILE(mnode) "\";\n";
+  t << "          }\n";
+  t << "          else if (imageNode.src.substring(l-17,l) == \"" FTV_ICON_FILE(plastnode) "\")\n";
+  t << "          {\n";
+  t << "            imageNode.src = \"" FTV_ICON_FILE(mlastnode) "\";\n";
+  t << "          }\n";
+  t << "        }\n";
+  t << "        folder.style.display = \"block\";\n";
+  t << "      }\n";
+  t << "    }\n";
+  t << "\n";
+  t << "    // End script hiding -->        \n";
+  t << "    </script>\n";
+}
+
+void FTVHelp::generateTreeViewInline(FTextStream &t)
+{
+  generateScript(t);
+  t << "    <div class=\"directory-alt\">\n";
+  t << "      <br/>\n";
+  t << "      <div style=\"display: block;\">\n";
+
+  generateTree(t,m_indentNodes[0],0);
+
+  t << "      </div>\n";
+  t << "    </div>\n";
+}
+
+
 void FTVHelp::generateTreeView()
 {
-  // Generate alternative index.html as a frame
-  QCString fileName=Config_getString("HTML_OUTPUT")+"/index"+Doxygen::htmlFileExtension;
+  QCString fileName;
   QFile f;
+  static bool searchEngine = Config_getBool("SEARCHENGINE");
+  static bool serverBasedSearch = Config_getBool("SERVER_BASED_SEARCH");
+  generateTreeViewImages();
+  
+  fileName=Config_getString("HTML_OUTPUT")+"/index"+Doxygen::htmlFileExtension;
   f.setName(fileName);
   if (!f.open(IO_WriteOnly))
   {
@@ -593,13 +661,11 @@ void FTVHelp::generateTreeView()
   }
   else
   {
-    QTextStream t(&f);
-#if QT_VERSION >= 200
-    t.setEncoding(QTextStream::Latin1);
-#endif
-    t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\">\n";
-    t << "<html><head>";
-    t << "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=UTF-8\">\n";
+    FTextStream t(&f);
+    //t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\">\n";
+    t << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Frameset//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd\">\n";
+    t << "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n<head>\n";
+    t << "<meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=UTF-8\"/>\n";
     t << "<title>"; 
     if (Config_getString("PROJECT_NAME").isEmpty())
     {
@@ -609,19 +675,21 @@ void FTVHelp::generateTreeView()
     {
       t << Config_getString("PROJECT_NAME");
     }
-    t << "</title></head>" << endl;
+    t << "</title>\n</head>" << endl;
     t << "<frameset cols=\"" << Config_getInt("TREEVIEW_WIDTH") << ",*\">" << endl;
-    t << "  <frame src=\"tree" << Doxygen::htmlFileExtension << "\" name=\"treefrm\">" << endl;
-    t << "  <frame src=\"main" << Doxygen::htmlFileExtension << "\" name=\"basefrm\">" << endl;
+    t << "  <frame src=\"tree" << Doxygen::htmlFileExtension << "\" name=\"treefrm\"/>" << endl;
+    t << "  <frame src=\"main" << Doxygen::htmlFileExtension << "\" name=\"basefrm\"/>" << endl;
     t << "  <noframes>" << endl;
+    t << "    <body>" << endl;
     t << "    <a href=\"main" << Doxygen::htmlFileExtension << "\">Frames are disabled. Click here to go to the main page.</a>" << endl;
+    t << "    </body>" << endl;
     t << "  </noframes>" << endl;
     t << "</frameset>" << endl;
     t << "</html>" << endl;
     f.close();
   }
 
-  // Generate tree view frame
+  // Generate tree view
   fileName=Config_getString("HTML_OUTPUT")+"/tree"+Doxygen::htmlFileExtension;
   f.setName(fileName);
   if (!f.open(IO_WriteOnly))
@@ -631,15 +699,23 @@ void FTVHelp::generateTreeView()
   }
   else
   {
-    QTextStream t(&f);
-#if QT_VERSION >= 200
-    t.setEncoding(QTextStream::Latin1);
-#endif
+    FTextStream t(&f);
+
+    //if (searchEngine)
+    //{
+    //  t << "<!-- This comment will put IE 6, 7 and 8 in quirks mode -->" << endl;
+    //}
+    t << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
     t << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
     t << "  <head>\n";
-    t << "    <meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=UTF-8\" />\n";
+    t << "    <meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=UTF-8\"/>\n";
     t << "    <meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />\n";
     t << "    <meta http-equiv=\"Content-Language\" content=\"en\" />\n";
+    if (searchEngine)
+    {
+      t << "    <link href=\"search/search.css\" rel=\"stylesheet\" type=\"text/css\"/>" << endl;
+      t << "    <script type=\"text/javaScript\" src=\"search/search.js\"></script>" << endl;
+    }
     t << "    <link rel=\"stylesheet\" href=\"";
     QCString cssname=Config_getString("HTML_STYLESHEET");
     if (cssname.isEmpty())
@@ -655,71 +731,64 @@ void FTVHelp::generateTreeView()
       }
       t << cssfi.fileName();
     }
-    t << "\">" << endl;
+    t << "\"/>" << endl;
     t << "    <title>TreeView</title>\n";
-    t << "    <script type=\"text/javascript\">\n";
-    t << "    <!-- // Hide script from old browsers\n";
-    t << "    \n";
-
-            /* User has clicked on a node (folder or +/-) in the tree */
-    t << "    function toggleFolder(id, imageNode) \n";
-    t << "    {\n";
-    t << "      var folder = document.getElementById(id);\n";
-    t << "      var l = imageNode.src.length;\n";
-                /* If the user clicks on the book icon, we move left one image so 
-                 * the code (below) will also adjust the '+' icon. 
-                 */
-    t << "      if (imageNode.src.substring(l-20,l)==\"" FTV_ICON_FILE(folderclosed) "\" || \n";
-    t << "          imageNode.src.substring(l-18,l)==\"" FTV_ICON_FILE(folderopen)  "\")\n";
-    t << "      {\n";
-    t << "        imageNode = imageNode.previousSibling;\n";
-    t << "        l = imageNode.src.length;\n";
-    t << "      }\n";
-    t << "      if (folder == null) \n";
-    t << "      {\n";
-    t << "      } \n";
-                /* Node controls a open section, we need to close it */
-    t << "      else if (folder.style.display == \"block\") \n";
-    t << "      {\n";
-    t << "        if (imageNode != null) \n";
-    t << "        {\n";
-    t << "          imageNode.nextSibling.src = \"" FTV_ICON_FILE(folderclosed) "\";\n";
-    t << "          if (imageNode.src.substring(l-13,l) == \"" FTV_ICON_FILE(mnode) "\")\n";
-    t << "          {\n";
-    t << "            imageNode.src = \"" FTV_ICON_FILE(pnode) "\";\n";
-    t << "          }\n";
-    t << "          else if (imageNode.src.substring(l-17,l) == \"" FTV_ICON_FILE(mlastnode) "\")\n";
-    t << "          {\n";
-    t << "            imageNode.src = \"" FTV_ICON_FILE(plastnode) "\";\n";
-    t << "          }\n";
-    t << "        }\n";
-    t << "        folder.style.display = \"none\";\n";
-    t << "      } \n";
-    t << "      else \n"; /* section is closed, we need to open it */
-    t << "      {\n";
-    t << "        if (imageNode != null) \n";
-    t << "        {\n";
-    t << "          imageNode.nextSibling.src = \"" FTV_ICON_FILE(folderopen) "\";\n";
-    t << "          if (imageNode.src.substring(l-13,l) == \"" FTV_ICON_FILE(pnode) "\")\n";
-    t << "          {\n";
-    t << "            imageNode.src = \"" FTV_ICON_FILE(mnode) "\";\n";
-    t << "          }\n";
-    t << "          else if (imageNode.src.substring(l-17,l) == \"" FTV_ICON_FILE(plastnode) "\")\n";
-    t << "          {\n";
-    t << "            imageNode.src = \"" FTV_ICON_FILE(mlastnode) "\";\n";
-    t << "          }\n";
-    t << "        }\n";
-    t << "        folder.style.display = \"block\";\n";
-    t << "      }\n";
-    t << "    }\n";
-    t << "\n";
-    t << "    // End script hiding -->        \n";
-    t << "    </script>\n";
+    generateScript(t);
     t << "  </head>\n";
     t << "\n";
-    t << "  <body class=\"ftvtree\">\n";
+    t << "  <body class=\"ftvtree\"";
+    if (searchEngine && !serverBasedSearch)
+    {
+      t << " onload='searchBox.OnSelectItem(0);'";
+    }
+    t << ">\n";
+    if (searchEngine)
+    {
+      t << "      <script type=\"text/javascript\"><!--\n";
+      t << "      var searchBox = new SearchBox(\"searchBox\", \"search\", true, '" 
+        << theTranslator->trSearch() << "');\n";
+      t << "      --></script>\n";
+      if (!serverBasedSearch)
+      {
+        t << "      <div class=\"tabsearch\">\n";
+        t << "        <div id=\"MSearchBox\" class=\"MSearchBoxInactive\">\n";
+        t << "          <span class=\"left\">\n";
+        t << "            <a id=\"MSearchClose\" href=\"javascript:searchBox.CloseResultsWindow()\">"
+          << "<img id=\"MSearchCloseImg\" border=\"0\" src=\"search/close.png\" alt=\"\"/></a>\n";
+        t << "            <input type=\"text\" id=\"MSearchField\" value=\"" 
+          << theTranslator->trSearch() << "\" accesskey=\"S\"\n";
+        t << "                   onfocus=\"searchBox.OnSearchFieldFocus(true)\" \n";
+        t << "                   onblur=\"searchBox.OnSearchFieldFocus(false)\" \n";
+        t << "                   onkeyup=\"searchBox.OnSearchFieldChange(event)\"/>\n";
+        t << "          </span><span class=\"right\">\n";
+        t << "            <img id=\"MSearchSelect\" src=\"search/mag_sel.png\"\n";
+        t << "                 onmouseover=\"return searchBox.OnSearchSelectShow()\"\n";
+        t << "                 onmouseout=\"return searchBox.OnSearchSelectHide()\"\n";
+        t << "                 alt=\"\"/>\n";
+        t << "          </span>\n";
+        t << "        </div>\n";
+        t << "      </div>\n";
+        HtmlGenerator::writeSearchFooter(t,QCString());
+      }
+      else
+      {
+        t << "      <div class=\"tabsearch\">\n";
+        t << "        <div id=\"MSearchBox\" class=\"MSearchBoxInactive\">\n";
+        t << "          <span class=\"left\">\n";
+        t << "            <form id=\"FSearchBox\" action=\"search.php\" method=\"get\" target=\"basefrm\">\n";
+        t << "              <img id=\"MSearchSelect\" src=\"search/mag.png\" alt=\"\"/>\n";
+        t << "              <input type=\"text\" id=\"MSearchField\" name=\"query\" value=\""
+          << theTranslator->trSearch() << "\" size=\"20\" accesskey=\"S\" \n";
+        t << "                     onfocus=\"searchBox.OnSearchFieldFocus(true)\" \n";
+        t << "                     onblur=\"searchBox.OnSearchFieldFocus(false)\"/>\n";
+        t << "            </form>\n";
+        t << "          </span><span class=\"right\"></span>\n";
+        t << "        </div>\n";
+        t << "      </div>\n";
+      }
+    }
     t << "    <div class=\"directory\">\n";
-    t << "      <h3>";
+    t << "      <h3 class=\"swap\"><span>";
     QCString &projName = Config_getString("PROJECT_NAME");
     if (projName.isEmpty())
     {
@@ -729,17 +798,18 @@ void FTVHelp::generateTreeView()
     {
       t << projName;
     }
-    t << "</h3>\n";
+    t << "</span></h3>\n";
     t << "      <div style=\"display: block;\">\n";
 
     generateTree(t,m_indentNodes[0],0);
 
     t << "      </div>\n";
     t << "    </div>\n";
+  
     t << "  </body>\n";
     t << "</html>\n";
+  
+    f.close();
   }
-  generateTreeViewImages();
-
 }
 
