@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -24,6 +24,7 @@
 #include <qdict.h>
 #include <qregexp.h>
 #include "qtextcodec.h"
+#include "sortdict.h"
 
 #include "htmlhelp.h"
 #include "config.h"
@@ -43,28 +44,15 @@ struct IndexField
   bool     reversed;
 };
 
-class IndexFieldList : public QList<IndexField>
+class IndexFieldSDict : public SDict<IndexField>
 {
   public:
+    IndexFieldSDict() : SDict<IndexField>(17) {}
+   ~IndexFieldSDict() {}
     int compareItems(GCI item1, GCI item2)
     {
       return stricmp(((IndexField *)item1)->name,((IndexField *)item2)->name);
     }
-   ~IndexFieldList() {}
-};
-
-class IndexFieldListIterator : public QListIterator<IndexField>
-{
-  public:
-    IndexFieldListIterator( const IndexFieldList &list) :
-       QListIterator<IndexField>(list) {}
-};
-
-class IndexFieldDict : public QDict<IndexField>
-{
-  public:
-    IndexFieldDict(int size) : QDict<IndexField>(size) {}
-   ~IndexFieldDict() {}
 };
 
 /*! A helper class for HtmlHelp that manages a two level index in 
@@ -78,24 +66,21 @@ class HtmlHelpIndex
     void addItem(const char *first,const char *second, 
                  const char *url, const char *anchor,
                  bool hasLink,bool reversed);
-    void writeFields(QTextStream &t);
+    void writeFields(FTextStream &t);
   private:
-    IndexFieldList *list;
-    IndexFieldDict *dict;   
+    IndexFieldSDict *dict;   
 };
 
 /*! Constructs a new HtmlHelp index */
 HtmlHelpIndex::HtmlHelpIndex()
 {
-  list = new IndexFieldList;
-  dict = new IndexFieldDict(10007);
-  list->setAutoDelete(TRUE);
+  dict = new IndexFieldSDict;
+  dict->setAutoDelete(TRUE);
 }
 
 /*! Destroys the HtmlHelp index */
 HtmlHelpIndex::~HtmlHelpIndex()
 {
-  delete list;
   delete dict;
 }
 
@@ -132,8 +117,7 @@ void HtmlHelpIndex::addItem(const char *level1,const char *level2,
     f->anchor   = anchor;
     f->link     = hasLink;
     f->reversed = reversed;
-    list->inSort(f);
-    dict->insert(key,f);
+    dict->append(key,f);
   }
 }
 
@@ -163,9 +147,10 @@ void HtmlHelpIndex::addItem(const char *level1,const char *level2,
  *      b1     -> link to url#anchor 
  *  </pre>
  */
-void HtmlHelpIndex::writeFields(QTextStream &t)
+void HtmlHelpIndex::writeFields(FTextStream &t)
 {
-  IndexFieldListIterator ifli(*list);
+  dict->sort();
+  IndexFieldSDict::Iterator ifli(*dict);
   IndexField *f;
   QCString lastLevel1;
   bool level2Started=FALSE;
@@ -302,8 +287,13 @@ static QDict<QCString> s_languageDict;
 void HtmlHelp::initialize()
 {
   const char *str = Config_getString("CHM_INDEX_ENCODING");
-  if(!str) str = "Windows-1250";
+  if (!str) str = "CP1250"; // use safe and likely default
   m_fromUtf8 = portable_iconv_open(str,"UTF-8"); 
+  if (m_fromUtf8==(void *)(-1))
+  {
+    err("Error: unsupported character conversion for CHM_INDEX_ENCODING: '%s'->'UTF-8'\n", str);
+    exit(1);
+  }
 
   /* open the contents file */
   QCString fName = Config_getString("HTML_OUTPUT") + "/index.hhc";
@@ -315,7 +305,6 @@ void HtmlHelp::initialize()
   }
   /* Write the header of the contents file */
   cts.setDevice(cf);
-  cts.setEncoding(QTextStream::Latin1);
   cts << "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n"
          "<HTML><HEAD></HEAD><BODY>\n"
          "<OBJECT type=\"text/site properties\">\n"
@@ -333,7 +322,6 @@ void HtmlHelp::initialize()
   }
   /* Write the header of the contents file */
   kts.setDevice(kf);
-  kts.setEncoding(QTextStream::Latin1);
   kts << "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n"
          "<HTML><HEAD></HEAD><BODY>\n"
          "<OBJECT type=\"text/site properties\">\n"
@@ -456,15 +444,10 @@ void HtmlHelp::createProjectFile()
   QFile f(fName);
   if (f.open(IO_WriteOnly))
   {
-    QTextStream t(&f);
-#if QT_VERSION >= 200
-    t.setEncoding(QTextStream::Latin1);
-#endif
-
-   
+    FTextStream t(&f);
     
     QCString indexName="index"+Doxygen::htmlFileExtension;
-    if (usingTreeIndex()) indexName="main"+Doxygen::htmlFileExtension;
+    if (Config_getBool("GENERATE_TREEVIEW")) indexName="main"+Doxygen::htmlFileExtension;
     t << "[OPTIONS]\n";
     if (!Config_getString("CHM_FILE").isEmpty())
     {
@@ -500,14 +483,19 @@ void HtmlHelp::createProjectFile()
       t << s << endl;
       s = indexFiles.next();
     }
+    // items not found by the html help compiler scan.
     t << "tabs.css" << endl;
-    t << "tab_b.gif" << endl;
-    t << "tab_l.gif" << endl;
-    t << "tab_r.gif" << endl;
+    t << "tab_a.png" << endl;
+    t << "tab_b.png" << endl;
+    t << "tab_h.png" << endl;
+    t << "tab_s.png" << endl;
+    t << "nav_h.png" << endl;
+    t << "nav_f.png" << endl;
+    t << "bc_s.png" << endl;
     if (Config_getBool("HTML_DYNAMIC_SECTIONS"))
     {
-      t << "open.gif" << endl;
-      t << "closed.gif" << endl;
+      t << "open.png" << endl;
+      t << "closed.png" << endl;
     }
     f.close();
   }
@@ -639,21 +627,36 @@ void HtmlHelp::addContentsItem(bool isDir,
   cts << "</OBJECT>\n";
 }
 
-/*! Add an list item to the index file.
- *  \param level1 the main index of the item.
- *  \param level2 the sub index of the item.
- *  \param contRef the output file refering to the container.
- *  \param memRef  the output file containing to the member documentation.
- *  \param anchor  the anchor of the item.
- *  \param md      the member definition corresponding to this item.
- *  \sa HtmlHelpIndex
- */
-void HtmlHelp::addIndexItem(const char *level1, const char *level2,
-                            const char *contRef, const char *memRef, 
-                            const char *anchor,const MemberDef *md)
+
+void HtmlHelp::addIndexItem(Definition *context,MemberDef *md,
+                            const char *word)
 {
-  (void)md;
-  index->addItem(level1,level2,contRef,anchor,TRUE,FALSE);
-  index->addItem(level2,level1,memRef,anchor,TRUE,TRUE);
+  if (md)
+  {
+    static bool separateMemberPages = Config_getBool("SEPARATE_MEMBER_PAGES");
+    if (context==0) // global member
+    {
+      if (md->getGroupDef())
+        context = md->getGroupDef();
+      else if (md->getFileDef())
+        context = md->getFileDef();
+    }
+    if (context==0) return; // should not happen
+
+    QCString cfname  = md->getOutputFileBase();
+    QCString cfiname = context->getOutputFileBase();
+    QCString level1  = context->name();
+    QCString level2  = md->name();
+    QCString contRef = separateMemberPages ? cfname : cfiname;
+    QCString memRef  = cfname;
+    QCString anchor  = md->anchor();
+    index->addItem(level1,level2,contRef,anchor,TRUE,FALSE);
+    index->addItem(level2,level1,memRef,anchor,TRUE,TRUE);
+  }
+  else if (context)
+  {
+    QCString level1  = word ? QCString(word) : context->name();
+    index->addItem(level1,0,context->getOutputFileBase(),0,TRUE,FALSE);
+  }
 }
 
