@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,9 +28,9 @@
 #include "msc.h"
 #include "htmlattrib.h"
 
-static QString escapeLabelName(const char *s)
+static QCString escapeLabelName(const char *s)
 {
-  QString result;
+  QCString result;
   const char *p=s;
   char c;
   while ((c=*p++))
@@ -73,9 +73,9 @@ static int rowspan(DocHtmlCell *cell)
   return retval;
 }
 
-QString LatexDocVisitor::escapeMakeIndexChars(const char *s)
+QCString LatexDocVisitor::escapeMakeIndexChars(const char *s)
 {
-  QString result;
+  QCString result;
   const char *p=s;
   char str[2]; str[1]=0;
   char c;
@@ -96,7 +96,7 @@ QString LatexDocVisitor::escapeMakeIndexChars(const char *s)
 }
 
 
-LatexDocVisitor::LatexDocVisitor(QTextStream &t,CodeOutputInterface &ci,
+LatexDocVisitor::LatexDocVisitor(FTextStream &t,CodeOutputInterface &ci,
                                  const char *langExt,bool insideTabbing) 
   : DocVisitor(DocVisitor_Latex), m_t(t), m_ci(ci), m_insidePre(FALSE), 
     m_insideItem(FALSE), m_hide(FALSE), m_insideTabbing(insideTabbing),
@@ -185,7 +185,7 @@ void LatexDocVisitor::visit(DocSymbol *s)
                                m_t << "\\^{" << s->letter() << "}"; 
                              break;
     case DocSymbol::Slash:   if (tolower(s->letter())=='o')
-                               m_t << "\\" << s->letter();
+                               m_t << "{\\" << s->letter() << "}";
                              else
                                m_t << s->letter();
                              break;
@@ -197,7 +197,7 @@ void LatexDocVisitor::visit(DocSymbol *s)
     case DocSymbol::AElig:   m_t << "{\\AE}"; break;
     case DocSymbol::Aelig:   m_t << "{\\ae}"; break;
     default:
-                             err("Error: unknown symbol found\n");
+                             err("error: unknown symbol found\n");
   }
 }
 
@@ -233,13 +233,13 @@ void LatexDocVisitor::visit(DocStyleChange *s)
   switch (s->style())
   {
     case DocStyleChange::Bold:
-      if (s->enable()) m_t << "{\\bf ";      else m_t << "}";
+      if (s->enable()) m_t << "{\\bfseries ";      else m_t << "}";
       break;
     case DocStyleChange::Italic:
-      if (s->enable()) m_t << "{\\em ";     else m_t << "\\/}";
+      if (s->enable()) m_t << "{\\itshape ";     else m_t << "\\/}";
       break;
     case DocStyleChange::Code:
-      if (s->enable()) m_t << "{\\tt ";   else m_t << "}";
+      if (s->enable()) m_t << "{\\ttfamily ";   else m_t << "}";
       break;
     case DocStyleChange::Subscript:
       if (s->enable()) m_t << "$_{\\mbox{";    else m_t << "}}$ ";
@@ -256,13 +256,13 @@ void LatexDocVisitor::visit(DocStyleChange *s)
     case DocStyleChange::Preformatted:
       if (s->enable()) 
       {
-        m_t << "\\small\\begin{alltt}";
+        m_t << "\n\\begin{DoxyPre}";
         m_insidePre=TRUE;
       }
       else
       {
         m_insidePre=FALSE;
-        m_t << "\\end{alltt}\n\\normalsize " << endl;
+        m_t << "\\end{DoxyPre}\n";
       }
       break;
     case DocStyleChange::Div:  /* HTML only */ break;
@@ -272,35 +272,35 @@ void LatexDocVisitor::visit(DocStyleChange *s)
 
 void LatexDocVisitor::visit(DocVerbatim *s)
 {
-  static bool latexSourceCode = Config_getBool("LATEX_SOURCE_CODE");
+  //static bool latexSourceCode = Config_getBool("LATEX_SOURCE_CODE");
   if (m_hide) return;
   switch(s->type())
   {
     case DocVerbatim::Code: 
-      if (latexSourceCode)
+      //if (latexSourceCode)
+      //{
+      //  m_t << "\n\n\\begin{footnotesize}\\begin{alltt}" << endl; 
+      //}
+      //else
       {
-        m_t << "\n\n\\begin{footnotesize}\\begin{alltt}" << endl; 
-      }
-      else
-      {
-        m_t << "\n\n\\begin{Code}\\begin{verbatim}"; 
+        m_t << "\n\\begin{DoxyCode}\n";
       }
       Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,s->context(),s->text().latin1(),
+                            ->parseCode(m_ci,s->context(),s->text(),
                                         s->isExample(),s->exampleFile());
-      if (latexSourceCode)
+      //if (latexSourceCode)
+      //{
+      //  m_t << "\\end{alltt}\\end{footnotesize}" << endl; 
+      //}
+      //else
       {
-        m_t << "\\end{alltt}\\end{footnotesize}" << endl; 
-      }
-      else
-      {
-        m_t << "\\end{verbatim}\n\\end{Code}\n" << endl; 
+        m_t << "\\end{DoxyCode}\n";
       }
       break;
     case DocVerbatim::Verbatim: 
-      m_t << "\n\n\\begin{footnotesize}\\begin{verbatim}"; 
+      m_t << "\\begin{DoxyVerb}";
       m_t << s->text();
-      m_t << "\\end{verbatim}\n\\end{footnotesize}" << endl; 
+      m_t << "\\end{DoxyVerb}\n";
       break;
     case DocVerbatim::HtmlOnly: 
     case DocVerbatim::XmlOnly: 
@@ -384,33 +384,33 @@ void LatexDocVisitor::visit(DocInclude *inc)
   {
     case DocInclude::IncWithLines:
       { 
-         m_t << "\n\n\\begin{DocInclude}\\begin{verbatim}"; 
+         m_t << "\n\\begin{DoxyCodeInclude}\n";
          QFileInfo cfi( inc->file() );
          FileDef fd( cfi.dirPath(), cfi.fileName() );
          Doxygen::parserManager->getParser(inc->extension())
                                ->parseCode(m_ci,inc->context(),
-                                           inc->text().latin1(),
+                                           inc->text(),
                                            inc->isExample(),
                                            inc->exampleFile(), &fd);
-         m_t << "\\end{verbatim}\n\\end{DocInclude}" << endl; 
+         m_t << "\\end{DoxyCodeInclude}" << endl;
       }
       break;    
     case DocInclude::Include: 
-      m_t << "\n\n\\begin{DocInclude}\\begin{verbatim}"; 
+      m_t << "\n\\begin{DoxyCodeInclude}\n";
       Doxygen::parserManager->getParser(inc->extension())
                             ->parseCode(m_ci,inc->context(),
-                                        inc->text().latin1(),inc->isExample(),
+                                        inc->text(),inc->isExample(),
                                         inc->exampleFile());
-      m_t << "\\end{verbatim}\n\\end{DocInclude}" << endl; 
+      m_t << "\\end{DoxyCodeInclude}\n";
       break;
     case DocInclude::DontInclude: 
       break;
     case DocInclude::HtmlInclude: 
       break;
     case DocInclude::VerbInclude: 
-      m_t << "\n\n\\begin{VerbInclude}\\begin{verbatim}"; 
+      m_t << "\n\\begin{DoxyVerbInclude}\n";
       m_t << inc->text();
-      m_t << "\\end{verbatim}\n\\end{VerbInclude}" << endl; 
+      m_t << "\\end{DoxyVerbInclude}\n";
       break;
   }
 }
@@ -421,7 +421,7 @@ void LatexDocVisitor::visit(DocIncOperator *op)
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
   if (op->isFirst()) 
   {
-    if (!m_hide) m_t << "\n\n\\begin{DocInclude}\\begin{verbatim}"; 
+    if (!m_hide) m_t << "\n\\begin{DoxyCodeInclude}\n";
     pushEnabled();
     m_hide = TRUE;
   }
@@ -431,7 +431,7 @@ void LatexDocVisitor::visit(DocIncOperator *op)
     if (!m_hide) 
     {
       Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,op->context(),op->text().latin1(),
+                            ->parseCode(m_ci,op->context(),op->text(),
                                         op->isExample(),op->exampleFile());
     }
     pushEnabled();
@@ -440,7 +440,7 @@ void LatexDocVisitor::visit(DocIncOperator *op)
   if (op->isLast())  
   {
     popEnabled();
-    if (!m_hide) m_t << "\\end{verbatim}\n\\end{DocInclude}" << endl; 
+    if (!m_hide) m_t << "\n\\end{DoxyCodeInclude}\n";
   }
   else
   {
@@ -462,6 +462,10 @@ void LatexDocVisitor::visit(DocIndexEntry *i)
   m_t << "}}";
 }
 
+void LatexDocVisitor::visit(DocSimpleSectSep *)
+{
+}
+
 //--------------------------------------
 // visitor functions for compound nodes
 //--------------------------------------
@@ -471,11 +475,11 @@ void LatexDocVisitor::visitPre(DocAutoList *l)
   if (m_hide) return;
   if (l->isEnumList())
   {
-    m_t << "\\begin{enumerate}" << endl;
+    m_t << "\n\\begin{DoxyEnumerate}";
   }
   else
   {
-    m_t << "\\begin{itemize}" << endl;
+    m_t << "\n\\begin{DoxyItemize}";
   }
 }
 
@@ -484,18 +488,18 @@ void LatexDocVisitor::visitPost(DocAutoList *l)
   if (m_hide) return;
   if (l->isEnumList())
   {
-    m_t << "\\end{enumerate}" << endl;
+    m_t << "\n\\end{DoxyEnumerate}";
   }
   else
   {
-    m_t << "\\end{itemize}" << endl;
+    m_t << "\n\\end{DoxyItemize}";
   }
 }
 
 void LatexDocVisitor::visitPre(DocAutoListItem *)
 {
   if (m_hide) return;
-  m_t << "\\item ";
+  m_t << "\n\\item ";
 }
 
 void LatexDocVisitor::visitPost(DocAutoListItem *) 
@@ -527,46 +531,77 @@ void LatexDocVisitor::visitPost(DocRoot *)
 void LatexDocVisitor::visitPre(DocSimpleSect *s)
 {
   if (m_hide) return;
-  m_t << "\\begin{Desc}\n\\item[";
   switch(s->type())
   {
-    case DocSimpleSect::See: 
-      filter(theTranslator->trSeeAlso()); break;
-    case DocSimpleSect::Return: 
-      filter(theTranslator->trReturns()); break;
-    case DocSimpleSect::Author: 
-      filter(theTranslator->trAuthor(TRUE,TRUE)); break;
-    case DocSimpleSect::Authors: 
-      filter(theTranslator->trAuthor(TRUE,FALSE)); break;
-    case DocSimpleSect::Version: 
-      filter(theTranslator->trVersion()); break;
-    case DocSimpleSect::Since: 
-      filter(theTranslator->trSince()); break;
-    case DocSimpleSect::Date: 
-      filter(theTranslator->trDate()); break;
-    case DocSimpleSect::Note: 
-      filter(theTranslator->trNote()); break;
+    case DocSimpleSect::See:
+      m_t << "\\begin{DoxySeeAlso}{";
+      filter(theTranslator->trSeeAlso());
+      break;
+    case DocSimpleSect::Return:
+      m_t << "\\begin{DoxyReturn}{";
+      filter(theTranslator->trReturns());
+      break;
+    case DocSimpleSect::Author:
+      m_t << "\\begin{DoxyAuthor}{";
+      filter(theTranslator->trAuthor(TRUE,TRUE));
+      break;
+    case DocSimpleSect::Authors:
+      m_t << "\\begin{DoxyAuthor}{";
+      filter(theTranslator->trAuthor(TRUE,FALSE));
+      break;
+    case DocSimpleSect::Version:
+      m_t << "\\begin{DoxyVersion}{";
+      filter(theTranslator->trVersion());
+      break;
+    case DocSimpleSect::Since:
+      m_t << "\\begin{DoxySince}{";
+      filter(theTranslator->trSince());
+      break;
+    case DocSimpleSect::Date:
+      m_t << "\\begin{DoxyDate}{";
+      filter(theTranslator->trDate());
+      break;
+    case DocSimpleSect::Note:
+      m_t << "\\begin{DoxyNote}{";
+      filter(theTranslator->trNote());
+      break;
     case DocSimpleSect::Warning:
-      filter(theTranslator->trWarning()); break;
+      m_t << "\\begin{DoxyWarning}{";
+      filter(theTranslator->trWarning());
+      break;
     case DocSimpleSect::Pre:
-      filter(theTranslator->trPrecondition()); break;
+      m_t << "\\begin{DoxyPrecond}{";
+      filter(theTranslator->trPrecondition());
+      break;
     case DocSimpleSect::Post:
-      filter(theTranslator->trPostcondition()); break;
+      m_t << "\\begin{DoxyPostcond}{";
+      filter(theTranslator->trPostcondition());
+      break;
     case DocSimpleSect::Invar:
-      filter(theTranslator->trInvariant()); break;
+      m_t << "\\begin{DoxyInvariant}{";
+      filter(theTranslator->trInvariant());
+      break;
     case DocSimpleSect::Remark:
-      filter(theTranslator->trRemarks()); break;
+      m_t << "\\begin{DoxyRemark}{";
+      filter(theTranslator->trRemarks());
+      break;
     case DocSimpleSect::Attention:
-      filter(theTranslator->trAttention()); break;
-    case DocSimpleSect::User: break;
-    case DocSimpleSect::Rcs: break;
+      m_t << "\\begin{DoxyAttention}{";
+      filter(theTranslator->trAttention());
+      break;
+    case DocSimpleSect::User:
+      m_t << "\\begin{DoxyParagraph}{";
+      break;
+    case DocSimpleSect::Rcs: 
+      m_t << "\\begin{DoxyParagraph}{";
+      break;
     case DocSimpleSect::Unknown:  break;
   }
 
   // special case 1: user defined title
   if (s->type()!=DocSimpleSect::User && s->type()!=DocSimpleSect::Rcs)
   {
-    m_t << ":]";
+    m_t << "}\n";
   }
   else
   {
@@ -574,10 +609,62 @@ void LatexDocVisitor::visitPre(DocSimpleSect *s)
   }
 }
 
-void LatexDocVisitor::visitPost(DocSimpleSect *)
+void LatexDocVisitor::visitPost(DocSimpleSect *s)
 {
   if (m_hide) return;
-  m_t << "\\end{Desc}" << endl;
+  switch(s->type())
+  {
+    case DocSimpleSect::See:
+      m_t << "\n\\end{DoxySeeAlso}\n";
+      break;
+    case DocSimpleSect::Return:
+      m_t << "\n\\end{DoxyReturn}\n";
+      break;
+    case DocSimpleSect::Author:
+      m_t << "\n\\end{DoxyAuthor}\n";
+      break;
+    case DocSimpleSect::Authors:
+      m_t << "\n\\end{DoxyAuthor}\n";
+      break;
+    case DocSimpleSect::Version:
+      m_t << "\n\\end{DoxyVersion}\n";
+      break;
+    case DocSimpleSect::Since:
+      m_t << "\n\\end{DoxySince}\n";
+      break;
+    case DocSimpleSect::Date:
+      m_t << "\n\\end{DoxyDate}\n";
+      break;
+    case DocSimpleSect::Note:
+      m_t << "\n\\end{DoxyNote}\n";
+      break;
+    case DocSimpleSect::Warning:
+      m_t << "\n\\end{DoxyWarning}\n";
+      break;
+    case DocSimpleSect::Pre:
+      m_t << "\n\\end{DoxyPrecond}\n";
+      break;
+    case DocSimpleSect::Post:
+      m_t << "\n\\end{DoxyPostcond}\n";
+      break;
+    case DocSimpleSect::Invar:
+      m_t << "\n\\end{DoxyInvariant}\n";
+      break;
+    case DocSimpleSect::Remark:
+      m_t << "\n\\end{DoxyRemark}\n";
+      break;
+    case DocSimpleSect::Attention:
+      m_t << "\n\\end{DoxyAttention}\n";
+      break;
+    case DocSimpleSect::User:
+      m_t << "\n\\end{DoxyParagraph}\n";
+      break;
+    case DocSimpleSect::Rcs: 
+      m_t << "\n\\end{DoxyParagraph}\n";
+      break;
+    default:
+      break;
+  }
 }
 
 void LatexDocVisitor::visitPre(DocTitle *)
@@ -588,19 +675,19 @@ void LatexDocVisitor::visitPost(DocTitle *)
 {
   if (m_hide) return;
   m_insideItem=FALSE;
-  m_t << "]";
+  m_t << "}\n";
 }
 
 void LatexDocVisitor::visitPre(DocSimpleList *)
 {
   if (m_hide) return;
-  m_t << "\\begin{itemize}" << endl;
+  m_t << "\\begin{DoxyItemize}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocSimpleList *)
 {
   if (m_hide) return;
-  m_t << "\\end{itemize}" << endl;
+  m_t << "\\end{DoxyItemize}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocSimpleListItem *)
@@ -633,24 +720,24 @@ void LatexDocVisitor::visitPre(DocHtmlList *s)
 {
   if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
-    m_t << "\\begin{enumerate}" << endl; 
+    m_t << "\n\\begin{DoxyEnumerate}";
   else 
-    m_t << "\\begin{itemize}" << endl;
+    m_t << "\n\\begin{DoxyItemize}";
 }
 
 void LatexDocVisitor::visitPost(DocHtmlList *s) 
 {
   if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
-    m_t << "\\end{enumerate}" << endl; 
+    m_t << "\n\\end{DoxyEnumerate}";
   else 
-    m_t << "\\end{itemize}" << endl;
+    m_t << "\n\\end{DoxyItemize}";
 }
 
 void LatexDocVisitor::visitPre(DocHtmlListItem *)
 {
   if (m_hide) return;
-  m_t << "\\item ";
+  m_t << "\n\\item ";
 }
 
 void LatexDocVisitor::visitPost(DocHtmlListItem *) 
@@ -672,19 +759,19 @@ void LatexDocVisitor::visitPost(DocHtmlListItem *)
 void LatexDocVisitor::visitPre(DocHtmlDescList *)
 {
   if (m_hide) return;
-  m_t << "\\begin{description}" << endl;
+  m_t << "\n\\begin{DoxyDescription}";
 }
 
 void LatexDocVisitor::visitPost(DocHtmlDescList *) 
 {
   if (m_hide) return;
-  m_t << "\\end{description}" << endl;
+  m_t << "\n\\end{DoxyDescription}";
 }
 
 void LatexDocVisitor::visitPre(DocHtmlDescTitle *)
 {
   if (m_hide) return;
-  m_t << "\\item[";
+  m_t << "\n\\item[";
   m_insideItem=TRUE;
 }
 
@@ -806,14 +893,15 @@ void LatexDocVisitor::visitPost(DocHtmlCell *c)
 void LatexDocVisitor::visitPre(DocInternal *)
 {
   if (m_hide) return;
-  m_t << "\\begin{Desc}" << endl 
-    << "\\item["; filter(theTranslator->trForInternalUseOnly()); m_t << "]" << endl;
+  m_t << "\\begin{DoxyInternal}{";
+  filter(theTranslator->trForInternalUseOnly());
+  m_t << "}\n";
 }
 
 void LatexDocVisitor::visitPost(DocInternal *) 
 {
   if (m_hide) return;
-  m_t << "\\end{Desc}" << endl;
+  m_t << "\\end{DoxyInternal}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocHRef *href)
@@ -853,14 +941,14 @@ void LatexDocVisitor::visitPre(DocImage *img)
     if (m_hide) return;
     if (img->hasCaption())
     {
-      m_t << "\\begin{Image}" << endl;
-      m_t << "\\begin{center}" << endl;
+      m_t << "\n\\begin{DoxyImage}\n";
     }
     else
     {
-      m_t << "\\begin{ImageNoCaption}\\mbox{";
+      m_t << "\n\\begin{DoxyImageNoCaption}\n"
+             "  \\mbox{";
     }
-    QString gfxName = img->name();
+    QCString gfxName = img->name();
     if (gfxName.right(4)==".eps" || gfxName.right(4)==".pdf")
     {
       gfxName=gfxName.left(gfxName.length()-4);
@@ -877,7 +965,7 @@ void LatexDocVisitor::visitPre(DocImage *img)
     m_t << "{" << gfxName << "}";
     if (img->hasCaption())
     {
-      m_t << "\\caption{";
+      m_t << "\n\\caption{";
     }
   }
   else // other format -> skip
@@ -892,14 +980,13 @@ void LatexDocVisitor::visitPost(DocImage *img)
   if (img->type()==DocImage::Latex)
   {
     if (m_hide) return;
-    m_t << "}" << endl; // end mbox or caption
+    m_t << "}\n"; // end mbox or caption
     if (img->hasCaption())
     {
-      m_t << "\\end{center}" << endl;
-      m_t << "\\end{Image}" << endl;
+      m_t << "\\end{DoxyImage}\n";
     }
     else{
-      m_t << "\\end{ImageNoCaption}" << endl;
+      m_t << "\\end{DoxyImageNoCaption}\n";
     }
   }
   else // other format
@@ -919,7 +1006,17 @@ void LatexDocVisitor::visitPost(DocDotFile *df)
   if (m_hide) return;
   endDotFile(df->hasCaption());
 }
+void LatexDocVisitor::visitPre(DocMscFile *df)
+{
+  if (m_hide) return;
+  startMscFile(df->file(),df->width(),df->height(),df->hasCaption());
+}
 
+void LatexDocVisitor::visitPost(DocMscFile *df) 
+{
+  if (m_hide) return;
+  endMscFile(df->hasCaption());
+}
 void LatexDocVisitor::visitPre(DocLink *lnk)
 {
   if (m_hide) return;
@@ -962,20 +1059,20 @@ void LatexDocVisitor::visitPre(DocSecRefList *)
   if (m_hide) return;
   m_t << "\\footnotesize" << endl;
   m_t << "\\begin{multicols}{2}" << endl;
-  m_t << "\\begin{CompactList}" << endl;
+  m_t << "\\begin{DoxyCompactList}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocSecRefList *) 
 {
   if (m_hide) return;
-  m_t << "\\end{CompactList}" << endl;
+  m_t << "\\end{DoxyCompactList}" << endl;
   m_t << "\\end{multicols}" << endl;
   m_t << "\\normalsize" << endl;
 }
 
 //void LatexDocVisitor::visitPre(DocLanguage *l)
 //{
-//  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+//  QCString langId = Config_getEnum("OUTPUT_LANGUAGE");
 //  if (l->id().lower()!=langId.lower())
 //  {
 //    pushEnabled();
@@ -985,7 +1082,7 @@ void LatexDocVisitor::visitPost(DocSecRefList *)
 //
 //void LatexDocVisitor::visitPost(DocLanguage *l) 
 //{
-//  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+//  QCString langId = Config_getEnum("OUTPUT_LANGUAGE");
 //  if (l->id().lower()!=langId.lower())
 //  {
 //    popEnabled();
@@ -995,55 +1092,117 @@ void LatexDocVisitor::visitPost(DocSecRefList *)
 void LatexDocVisitor::visitPre(DocParamSect *s)
 {
   if (m_hide) return;
-  m_t << "\\begin{Desc}" << endl;
-  m_t << "\\item[";
+  bool hasInOutSpecs = s->hasInOutSpecifier();
+  bool hasTypeSpecs  = s->hasTypeSpecifier();
   switch(s->type())
   {
-    case DocParamSect::Param: 
-      filter(theTranslator->trParameters()); break;
-    case DocParamSect::RetVal: 
-      filter(theTranslator->trReturnValues()); break;
+    case DocParamSect::Param:
+      m_t << "\n\\begin{DoxyParams}";
+      if      (hasInOutSpecs && hasTypeSpecs) m_t << "[2]"; // 2 extra cols
+      else if (hasInOutSpecs || hasTypeSpecs) m_t << "[1]"; // 1 extra col
+      m_t << "{";
+      filter(theTranslator->trParameters());
+      break;
+    case DocParamSect::RetVal:
+      m_t << "\n\\begin{DoxyRetVals}{";
+      filter(theTranslator->trReturnValues());
+      break;
     case DocParamSect::Exception: 
-      filter(theTranslator->trExceptions()); break;
+      m_t << "\n\\begin{DoxyExceptions}{";
+      filter(theTranslator->trExceptions());
+      break;
     case DocParamSect::TemplateParam: 
       /* TODO: add this 
       filter(theTranslator->trTemplateParam()); break;
       */
-      filter("Template Parameters"); break;
+      m_t << "\n\\begin{DoxyTemplParams}{";
+      filter("Template Parameters");
+      break;
     default:
       ASSERT(0);
   }
-  m_t << ":]" << endl;
-  m_t << "\\begin{description}" << endl;
+  m_t << "}\n";
 }
 
-void LatexDocVisitor::visitPost(DocParamSect *)
+void LatexDocVisitor::visitPost(DocParamSect *s)
 {
   if (m_hide) return;
-  m_t << "\\end{description}" << endl;
-  m_t << "\\end{Desc}" << endl;
+  switch(s->type())
+  {
+    case DocParamSect::Param:
+      m_t << "\\end{DoxyParams}\n";
+      break;
+    case DocParamSect::RetVal:
+      m_t << "\\end{DoxyRetVals}\n";
+      break;
+    case DocParamSect::Exception: 
+      m_t << "\\end{DoxyExceptions}\n";
+      break;
+    case DocParamSect::TemplateParam: 
+      m_t << "\\end{DoxyTemplParams}\n";
+      break;
+    default:
+      ASSERT(0);
+  }
 }
 
 void LatexDocVisitor::visitPre(DocParamList *pl)
 {
   if (m_hide) return;
-  m_t << "\\item[";
-  if (pl->direction()!=DocParamSect::Unspecified)
+  DocParamSect::Type parentType = DocParamSect::Unknown;
+  DocParamSect *sect = 0;
+  if (pl->parent() && pl->parent()->kind()==DocNode::Kind_ParamSect)
   {
-    m_t << "\\mbox{";
-    if (pl->direction()==DocParamSect::In)
+    parentType = ((DocParamSect*)pl->parent())->type();
+    sect=(DocParamSect*)pl->parent();
+  }
+  bool useTable = parentType==DocParamSect::Param ||
+                  parentType==DocParamSect::RetVal ||
+                  parentType==DocParamSect::Exception ||
+                  parentType==DocParamSect::TemplateParam;
+  if (!useTable)
+  {
+    m_t << "\\item[";
+  }
+  if (sect && sect->hasInOutSpecifier())
+  {
+    if (pl->direction()!=DocParamSect::Unspecified)
     {
-      m_t << "$\\leftarrow$";
+      m_t << "\\mbox{\\tt ";
+      if (pl->direction()==DocParamSect::In)
+      {
+        m_t << "in";
+      }
+      else if (pl->direction()==DocParamSect::Out)
+      {
+        m_t << "out";
+      }
+      else if (pl->direction()==DocParamSect::InOut)
+      {
+        m_t << "in,out";
+      }
+      m_t << "} ";
     }
-    else if (pl->direction()==DocParamSect::Out)
+    if (useTable) m_t << " & ";
+  }
+  if (sect && sect->hasTypeSpecifier())
+  {
+    QListIterator<DocNode> li(pl->paramTypes());
+    DocNode *type;
+    bool first=TRUE;
+    for (li.toFirst();(type=li.current());++li)
     {
-      m_t << "$\\rightarrow$";
+      if (!first) m_t << " | "; else first=FALSE;
+      if (type->kind()==DocNode::Kind_Word)
+      {
+        visit((DocWord*)type); 
+      }
+      else if (type->kind()==DocNode::Kind_LinkedWord)
+      {
+        visit((DocLinkedWord*)type); 
+      }
     }
-    else if (pl->direction()==DocParamSect::InOut)
-    {
-      m_t << "$\\leftrightarrow$";
-    }
-    m_t << "} ";
+    if (useTable) m_t << " & ";
   }
   m_t << "{\\em ";
   //QStrListIterator li(pl->parameters());
@@ -1065,11 +1224,34 @@ void LatexDocVisitor::visitPre(DocParamList *pl)
     }
     m_insideItem=FALSE;
   }
-  m_t << "}]";
+  m_t << "}";
+  if (useTable)
+  {
+    m_t << " & ";
+  }
+  else
+  {
+    m_t << "]";
+  }
 }
 
-void LatexDocVisitor::visitPost(DocParamList *)
+void LatexDocVisitor::visitPost(DocParamList *pl)
 {
+  if (m_hide) return;
+  DocParamSect::Type parentType = DocParamSect::Unknown;
+  if (pl->parent() && pl->parent()->kind()==DocNode::Kind_ParamSect)
+  {
+    parentType = ((DocParamSect*)pl->parent())->type();
+  }
+  bool useTable = parentType==DocParamSect::Param ||
+                  parentType==DocParamSect::RetVal ||
+                  parentType==DocParamSect::Exception ||
+                  parentType==DocParamSect::TemplateParam;
+  if (useTable)
+  {
+    m_t << "\\\\" << endl
+        << "\\hline" << endl;
+  }
 }
 
 void LatexDocVisitor::visitPre(DocXRefItem *x)
@@ -1131,7 +1313,7 @@ void LatexDocVisitor::filter(const char *str)
   filterLatexString(m_t,str,m_insideTabbing,m_insidePre,m_insideItem);
 }
 
-void LatexDocVisitor::startLink(const QString &ref,const QString &file,const QString &anchor)
+void LatexDocVisitor::startLink(const QCString &ref,const QCString &file,const QCString &anchor)
 {
   if (ref.isEmpty() && Config_getBool("PDF_HYPERLINKS")) // internal PDF link 
   {
@@ -1162,7 +1344,7 @@ void LatexDocVisitor::startLink(const QString &ref,const QString &file,const QSt
   }
 }
 
-void LatexDocVisitor::endLink(const QString &ref,const QString &file,const QString &anchor)
+void LatexDocVisitor::endLink(const QCString &ref,const QCString &file,const QCString &anchor)
 {
   m_t << "}";
   if (ref.isEmpty() && !Config_getBool("PDF_HYPERLINKS"))
@@ -1188,13 +1370,13 @@ void LatexDocVisitor::popEnabled()
   delete v;
 }
 
-void LatexDocVisitor::startDotFile(const QString &fileName,
-                                   const QString &width,
-                                   const QString &height,
+void LatexDocVisitor::startDotFile(const QCString &fileName,
+                                   const QCString &width,
+                                   const QCString &height,
                                    bool hasCaption
                                   )
 {
-  QString baseName=fileName;
+  QCString baseName=fileName;
   int i;
   if ((i=baseName.findRev('/'))!=-1)
   {
@@ -1208,17 +1390,17 @@ void LatexDocVisitor::startDotFile(const QString &fileName,
   {
     baseName=baseName.left(baseName.length()-4);
   }
-  QString outDir = Config_getString("LATEX_OUTPUT");
-  QString name = fileName;
+  QCString outDir = Config_getString("LATEX_OUTPUT");
+  QCString name = fileName;
   writeDotGraphFromFile(name,outDir,baseName,EPS);
   if (hasCaption)
   {
-    m_t << "\\begin{Image}" << endl;
-    m_t << "\\begin{center}" << endl;
+    m_t << "\n\\begin{DoxyImage}\n";
   }
   else
   {
-    m_t << "\\begin{ImageNoCaption}\\mbox{";
+    m_t << "\n\\begin{DoxyImageNoCaption}\n"
+           "  \\mbox{";
   }
   m_t << "\\includegraphics";
   if (!width.isEmpty())
@@ -1229,46 +1411,114 @@ void LatexDocVisitor::startDotFile(const QString &fileName,
   {
     m_t << "[height=" << height << "]";
   }
+  else
+  {
+    m_t << "[width=\\textwidth]";
+  }
   m_t << "{" << baseName << "}";
 
   if (hasCaption)
   {
-    m_t << "\\caption{";
+    m_t << "\n\\caption{";
   }
 }
 
 void LatexDocVisitor::endDotFile(bool hasCaption)
 {
   if (m_hide) return;
-  m_t << "}" << endl; // end caption or mbox
+  m_t << "}\n"; // end caption or mbox
   if (hasCaption)
   {
-    m_t << "\\end{center}" << endl;
-    m_t << "\\end{Image}" << endl;
+    m_t << "\\end{DoxyImage}\n";
   }
   else
   {
-    m_t << "\\end{ImageNoCaption}" << endl;
+    m_t << "\\end{DoxyImageNoCaption}\n";
   }
 }
 
-void LatexDocVisitor::writeMscFile(const QString &baseName)
+void LatexDocVisitor::startMscFile(const QCString &fileName,
+                                   const QCString &width,
+                                   const QCString &height,
+                                   bool hasCaption
+                                  )
 {
-  QString outDir = Config_getString("LATEX_OUTPUT");
-  writeMscGraphFromFile(baseName,outDir,baseName,MSC_EPS);
-  m_t << "\\begin{ImageNoCaption}\\mbox{";
+  QCString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  } 
+  if (baseName.right(4)==".eps" || baseName.right(4)==".pdf")
+  {
+    baseName=baseName.left(baseName.length()-4);
+  }
+  if (baseName.right(4)==".dot")
+  {
+    baseName=baseName.left(baseName.length()-4);
+  }
+  QCString outDir = Config_getString("LATEX_OUTPUT");
+  QCString name = fileName;
+  writeMscGraphFromFile(name,outDir,baseName,MSC_EPS); 
+  if (hasCaption)
+  {
+    m_t << "\n\\begin{DoxyImage}\n";
+  }
+  else
+  {
+    m_t << "\n\\begin{DoxyImageNoCaption}\n"
+           "  \\mbox{";
+  }
   m_t << "\\includegraphics";
-  //if (!width.isEmpty())
-  //{
-  //  m_t << "[width=" << width << "]";
-  //}
-  //else if (!height.isEmpty())
-  //{
-  //  m_t << "[height=" << height << "]";
-  //}
+  if (!width.isEmpty())
+  {
+    m_t << "[width=" << width << "]";
+  }
+  else if (!height.isEmpty())
+  {
+    m_t << "[height=" << height << "]";
+  }
+  else
+  {
+    m_t << "[width=\\textwidth]";
+  }
   m_t << "{" << baseName << "}";
 
-  m_t << "}" << endl; // end mbox
-  m_t << "\\end{ImageNoCaption}" << endl;
+  if (hasCaption)
+  {
+    m_t << "\n\\caption{";
+  }
+}
+
+void LatexDocVisitor::endMscFile(bool hasCaption)
+{
+  if (m_hide) return;
+  m_t << "}\n"; // end caption or mbox
+  if (hasCaption)
+  {
+    m_t << "\\end{DoxyImage}\n";
+  }
+  else
+  {
+    m_t << "\\end{DoxyImageNoCaption}\n";
+  }
+}
+
+
+void LatexDocVisitor::writeMscFile(const QCString &baseName)
+{
+  QCString shortName = baseName;
+  int i;
+  if ((i=shortName.findRev('/'))!=-1)
+  {
+    shortName=shortName.right(shortName.length()-i-1);
+  } 
+  QCString outDir = Config_getString("LATEX_OUTPUT");
+  writeMscGraphFromFile(baseName,outDir,baseName,MSC_EPS);
+  m_t << "\n\\begin{DoxyImageNoCaption}"
+         "  \\mbox{\\includegraphics";
+  m_t << "{" << shortName << "}";
+  m_t << "}\n"; // end mbox
+  m_t << "\\end{DoxyImageNoCaption}\n";
 }
 
