@@ -2,7 +2,7 @@
  *
  * $Id: latexgen.cpp,v 1.58 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -92,7 +92,7 @@ void LatexGenerator::init()
   QCString latex_command = Config_getString("LATEX_CMD_NAME");
   QCString mkidx_command = Config_getString("MAKEINDEX_CMD_NAME");
   // end insertion by KONNO Akihisa <konno@researchers.jp> 2002-03-05
-  QTextStream t(&file);
+  FTextStream t(&file);
   if (!Config_getBool("USE_PDFLATEX")) // use plain old latex
   {
     t << "all: clean refman.dvi" << endl
@@ -161,12 +161,17 @@ void LatexGenerator::init()
 
   t << endl
     << "clean:" << endl
-    << "\trm -f *.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out refman.pdf" << endl;
+#if defined(_MSC_VER)
+    << "\tdel "  
+#else
+    << "\trm -f " 
+#endif
+    << "*.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out refman.pdf" << endl;
 
   createSubDirs(d);
 }
 
-static void writeDefaultHeaderPart1(QTextStream &t)
+static void writeDefaultHeaderPart1(FTextStream &t)
 {
   // part 1
 
@@ -182,22 +187,24 @@ static void writeDefaultHeaderPart1(QTextStream &t)
   t << "]{";
   if (Config_getBool("COMPACT_LATEX")) t << "article"; else t << "book";
   t << "}\n";
-  if (paperType=="a4wide") t << "\\usepackage{a4wide}\n";
+  // the next package is obsolete (see bug 563698)
+  //if (paperType=="a4wide") t << "\\usepackage{a4wide}\n";
   t << "\\usepackage{makeidx}\n"
-    "\\usepackage{fancyhdr}\n"
     "\\usepackage{graphicx}\n"
     "\\usepackage{multicol}\n"
     "\\usepackage{float}\n"
     "\\usepackage{listings}\n"
     "\\usepackage{color}\n"
+    "\\usepackage{ifthen}\n"
+    "\\usepackage[table]{xcolor}\n"
     "\\usepackage{textcomp}\n"
     "\\usepackage{alltt}\n"
     //"\\usepackage{ae,aecompl,aeguill}\n"
     ;
-  if (Config_getBool("USE_PDFLATEX"))
-  {
-    t << "\\usepackage{times}" << endl;
-  }
+  //if (Config_getBool("USE_PDFLATEX"))
+  //{
+  //  t << "\\usepackage{times}" << endl;
+  //}
   if (Config_getBool("PDF_HYPERLINKS")) 
   {
     t << "\\usepackage{ifpdf}" << endl
@@ -230,6 +237,9 @@ static void writeDefaultHeaderPart1(QTextStream &t)
     // if the command is empty, no output is needed.
     t << sLanguageSupportCommand << endl;
   }
+  t << "\\usepackage{mathptmx}\n";
+  t << "\\usepackage[scaled=.90]{helvet}\n";
+  t << "\\usepackage{courier}\n";
   t << "\\usepackage{doxygen}\n";
 
   // define option for listings
@@ -268,7 +278,7 @@ static void writeDefaultHeaderPart1(QTextStream &t)
 
 }
 
-static void writeDefaultHeaderPart2(QTextStream &t)
+static void writeDefaultHeaderPart2(FTextStream &t)
 {
   // part 2
   t << "}\\\\" << endl
@@ -276,7 +286,7 @@ static void writeDefaultHeaderPart2(QTextStream &t)
     << "{\\large ";
 }
 
-static void writeDefaultHeaderPart3(QTextStream &t)
+static void writeDefaultHeaderPart3(FTextStream &t)
 {
   // part 3
   t << " Doxygen " << versionString << "}\\\\" << endl
@@ -297,53 +307,393 @@ static void writeDefaultHeaderPart3(QTextStream &t)
   }
 }
 
-static void writeDefaultStyleSheetPart1(QTextStream &t)
+static void writeDefaultStyleSheetPart1(FTextStream &t)
 {
   // part 1
-  t << "\\NeedsTeXFormat{LaTeX2e}\n";
-  t << "\\ProvidesPackage{doxygen}\n";
-  t << "\\RequirePackage{calc}\n";
-  t << "\\RequirePackage{array}\n";
-  t << "\\RequirePackage{color}\n";
-  t << "\\pagestyle{fancyplain}\n";
-  //t << "\\addtolength{\\headwidth}{\\marginparsep}\n";
-  //t << "\\addtolength{\\headwidth}{\\marginparwidth}\n";
-  t << "\\newcommand{\\clearemptydoublepage}{\\newpage{\\pagestyle{empty}";
-  t << "\\cleardoublepage}}\n";
+  t << "\\NeedsTeXFormat{LaTeX2e}\n"
+       "\\ProvidesPackage{doxygen}\n\n";
+  t << "% Packages used by this style file\n"
+       "\\RequirePackage{alltt}\n"
+       "\\RequirePackage{array}\n"
+       "\\RequirePackage{calc}\n"
+       "\\RequirePackage{color}\n"
+       "\\RequirePackage{fancyhdr}\n"
+       "\\RequirePackage{longtable}\n"
+       "\\RequirePackage{verbatim}\n"
+       "\\RequirePackage{ifthen}\n"
+       "\\RequirePackage{xcolor}\n\n";
+
+  t << "% Use helvetica font instead of times roman\n"
+       "\\RequirePackage{helvet}\n"
+       "\\RequirePackage{sectsty}\n"
+       "\\RequirePackage{tocloft}\n"
+       "\\allsectionsfont{\\usefont{OT1}{phv}{bc}{n}\\selectfont}\n"
+       "\\renewcommand{\\cftchapfont}{%\n"
+       "  \\fontsize{11}{13}\\usefont{OT1}{phv}{bc}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftchappagefont}{%\n"
+       "  \\fontsize{11}{13}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftsecfont}{%\n"
+       "  \\fontsize{10}{12}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftsecpagefont}{%\n"
+       "  \\fontsize{10}{12}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftsubsecfont}{%\n"
+       "  \\fontsize{10}{12}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftsubsecpagefont}{%\n"
+       "  \\fontsize{10}{12}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftsubsubsecfont}{%\n"
+       "  \\fontsize{9}{11}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftsubsubsecpagefont}{%\n"
+       "  \\fontsize{9}{11}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftparafont}{%\n"
+       "  \\fontsize{9}{11}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cftparapagefont}{%\n"
+       "  \\fontsize{9}{11}\\usefont{OT1}{phv}{c}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\cfttoctitlefont}{%\n"
+       "  \\fontsize{20}{22}\\usefont{OT1}{phv}{b}{n}\\selectfont\n"
+       "}\n"
+       "\\renewcommand{\\rmdefault}{phv}\n"
+       "\\renewcommand{\\bfdefault}{bc}\n"
+       "\n\n";
+
+  t << "% Setup fancy headings\n"
+       "\\pagestyle{fancyplain}\n"
+       "\\newcommand{\\clearemptydoublepage}{%\n"
+       "  \\newpage{\\pagestyle{empty}\\cleardoublepage}%\n"
+       "}\n";
   if (!Config_getBool("COMPACT_LATEX")) 
-    t << "\\renewcommand{\\chaptermark}[1]{\\markboth{#1}{}}\n";
-  t << "\\renewcommand{\\sectionmark}[1]{\\markright{\\thesection\\ #1}}\n";
-  t << "\\lhead[\\fancyplain{}{\\bfseries\\thepage}]\n";
-  t << "        {\\fancyplain{}{\\bfseries\\rightmark}}\n";
-  t << "\\rhead[\\fancyplain{}{\\bfseries\\leftmark}]\n";
-  t << "        {\\fancyplain{}{\\bfseries\\thepage}}\n";
-  t << "\\rfoot[\\fancyplain{}{\\bfseries\\scriptsize ";
+    t << "\\renewcommand{\\chaptermark}[1]{%\n"
+         "  \\markboth{#1}{}%\n"
+         "}\n";
+  t << "\\renewcommand{\\sectionmark}[1]{%\n"
+       "  \\markright{\\thesection\\ #1}%\n"
+       "}\n";
+
+  //t << "\\lhead[\\fancyplain{}{\\bfseries\\thepage}]{%\n"
+  //     "  \\fancyplain{}{\\bfseries\\rightmark}%\n"
+  //     "}\n";
+  //t << "\\rhead[\\fancyplain{}{\\bfseries\\leftmark}]{%\n"
+  //     "  \\fancyplain{}{\\bfseries\\thepage}%\n"
+  //     "}\n";
+  //t << "\\rfoot[\\fancyplain{}{\\bfseries\\scriptsize%\n  ";
+  t << "\\fancyhead[LE]{\\fancyplain{}{\\bfseries\\thepage}}\n";
+  t << "\\fancyhead[CE]{\\fancyplain{}{}}\n";
+  t << "\\fancyhead[RE]{\\fancyplain{}{\\bfseries\\leftmark}}\n";
+  t << "\\fancyhead[LO]{\\fancyplain{}{\\bfseries\\rightmark}}\n";
+  t << "\\fancyhead[CO]{\\fancyplain{}{}}\n";
+  t << "\\fancyhead[RO]{\\fancyplain{}{\\bfseries\\thepage}}\n";
+
+  t << "\\fancyfoot[LE]{\\fancyplain{}{}}\n";
+  t << "\\fancyfoot[CE]{\\fancyplain{}{}}\n";
+  t << "\\fancyfoot[RE]{\\fancyplain{}{\\bfseries\\scriptsize ";
 }
 
-static void writeDefaultStyleSheetPart2(QTextStream &t)
-{
-  t << "\\lfoot[]{\\fancyplain{}{\\bfseries\\scriptsize ";
-}
-
-static void writeDefaultStyleSheetPart3(QTextStream &t)
+static void writeDefaultStyleSheetPart2(FTextStream &t)
 {
   t << "}}\n";
-  t << "\\cfoot{}\n";
-  t << "\\newenvironment{Code}\n";
-  t << "{\\footnotesize}\n";
-  t << "{\\normalsize}\n";
+  t << "\\fancyfoot[LO]{\\fancyplain{}{\\bfseries\\scriptsize ";
+  //t << "\\lfoot[]{\\fancyplain{}{\\bfseries\\scriptsize%\n  ";
+
+}
+
+static void writeDefaultStyleSheetPart3(FTextStream &t)
+{
+  static bool latexSourceCode = Config_getBool("LATEX_SOURCE_CODE");
+  t << "}}\n";
+  //t << "\\cfoot{}\n\n";
+  t << "\\fancyfoot[CO]{\\fancyplain{}{}}\n";
+  t << "\\fancyfoot[RO]{\\fancyplain{}{}}\n";
+
+  t << "%---------- Internal commands used in this style file ----------------\n\n";
+  t << "% Generic environment used by all paragraph-based environments defined\n"
+       "% below. Note that the command \\title{...} needs to be defined inside\n"
+       "% those environments!\n"
+       "\\newenvironment{DoxyDesc}[1]{%\n"
+       "  \\begin{list}{}%\n"
+       "  {%\n"
+       "    \\settowidth{\\labelwidth}{40pt}%\n"
+       "    \\setlength{\\leftmargin}{\\labelwidth}%\n"
+       "    \\setlength{\\parsep}{0pt}%\n"
+       "    \\setlength{\\itemsep}{-4pt}%\n"
+       "    \\renewcommand{\\makelabel}{\\entrylabel}%\n"
+       "  }%\n"
+       "  \\item[#1]%\n"
+       "}{%\n"
+       "  \\end{list}%\n"
+       "}\n\n";
+  t << "%---------- Commands used by doxygen LaTeX output generator ----------\n\n";
+  t << "% Used by <pre> ... </pre>\n"
+       "\\newenvironment{DoxyPre}{%\n"
+       "  \\small%\n"
+       "  \\begin{alltt}%\n"
+       "}{%\n"
+       "  \\end{alltt}%\n"
+       "  \\normalsize%\n"
+       "}\n\n";
+  t << "% Used by @code ... @endcode\n"
+       "\\newenvironment{DoxyCode}{%\n";
+  if (latexSourceCode)
+  {
+    t << "\n\n\\begin{footnotesize}\\begin{alltt}%" << endl;
+  }
+  else
+  {
+    t << "  \\footnotesize%\n"
+         "  \\verbatim%\n";
+  }
+  t << "}{%\n";
+  if (latexSourceCode)
+  {
+    t << "\\end{alltt}\\end{footnotesize}%" << endl; 
+  }
+  else
+  {
+    t << "  \\endverbatim%\n"
+         "  \\normalsize%\n";
+  }
+  t << "}\n\n";
+  t << "% Used by @example, @include, @includelineno and @dontinclude\n"
+       "\\newenvironment{DoxyCodeInclude}{%\n"
+       "  \\DoxyCode%\n"
+       "}{%\n"
+       "  \\endDoxyCode%\n"
+       "}\n\n";
+  t << "% Used by @verbatim ... @endverbatim\n"
+       "\\newenvironment{DoxyVerb}{%\n"
+       "  \\footnotesize%\n"
+       "  \\verbatim%\n"
+       "}{%\n"
+       "  \\endverbatim%\n"
+       "  \\normalsize%\n"
+       "}\n\n";
+  t << "% Used by @verbinclude\n"
+       "\\newenvironment{DoxyVerbInclude}{%\n"
+       "  \\DoxyVerb%\n"
+       "}{%\n"
+       "  \\endDoxyVerb%\n"
+       "}\n\n";
+  t << "% Used by numbered lists (using '-#' or <ol> ... </ol>)\n"
+       "\\newenvironment{DoxyEnumerate}{%\n"
+       "  \\enumerate%\n"
+       "}{%\n"
+       "  \\endenumerate%\n"
+       "}\n\n";
+  t << "% Used by bullet lists (using '-', @li, @arg, or <ul> ... </ul>)\n"
+       "\\newenvironment{DoxyItemize}{%\n"
+       "  \\itemize%\n"
+       "}{%\n"
+       "  \\enditemize%\n"
+       "}\n\n";
+  t << "% Used by description lists (using <dl> ... </dl>)\n"
+       "\\newenvironment{DoxyDescription}{%\n"
+       "  \\description%\n"
+       "}{%\n"
+       "  \\enddescription%\n"
+       "}\n\n";
+  t << "% Used by @image, @dotfile, and @dot ... @enddot\n"
+       "% (only if caption is specified)\n"
+       "\\newenvironment{DoxyImage}{%\n"
+       "  \\begin{figure}[H]%\n"
+       "  \\begin{center}%\n"
+       "}{%\n"
+       "  \\end{center}%\n"
+       "  \\end{figure}%\n"
+       "}\n\n";
+  t << "% Used by @image, @dotfile, @dot ... @enddot, and @msc ... @endmsc\n"
+       "% (only if no caption is specified)\n"
+       "\\newenvironment{DoxyImageNoCaption}{%\n"
+       "}{%\n"
+       "}\n\n";
+  t << "% Used by @attention\n"
+       "\\newenvironment{DoxyAttention}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @author and @authors\n"
+       "\\newenvironment{DoxyAuthor}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @date\n"
+       "\\newenvironment{DoxyDate}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @invariant\n"
+       "\\newenvironment{DoxyInvariant}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @note\n"
+       "\\newenvironment{DoxyNote}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @post\n"
+       "\\newenvironment{DoxyPostcond}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @pre\n"
+       "\\newenvironment{DoxyPrecond}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @remark\n"
+       "\\newenvironment{DoxyRemark}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @return\n"
+       "\\newenvironment{DoxyReturn}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @since\n"
+       "\\newenvironment{DoxySince}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @see\n"
+       "\\newenvironment{DoxySeeAlso}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @version\n"
+       "\\newenvironment{DoxyVersion}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @warning\n"
+       "\\newenvironment{DoxyWarning}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "}{%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by @internal\n"
+       "\\newenvironment{DoxyInternal}[1]{%\n"
+       "  \\paragraph*{#1}%\n"
+       "}{%\n"
+       "}\n\n";
+  t << "% Used by @par and @paragraph\n"
+       "\\newenvironment{DoxyParagraph}[1]{%\n"
+       "  \\begin{list}{}%\n"
+       "  {%\n"
+       "    \\settowidth{\\labelwidth}{40pt}%\n"
+       "    \\setlength{\\leftmargin}{\\labelwidth}%\n"
+       "    \\setlength{\\parsep}{0pt}%\n"
+       "    \\setlength{\\itemsep}{-4pt}%\n"
+       "    \\renewcommand{\\makelabel}{\\entrylabel}%\n"
+       "  }%\n"
+       "  \\item[#1]%\n"
+       "}{%\n"
+       "  \\end{list}%\n"
+       "}\n\n";
+  t << "% Used by parameter lists\n"
+       "\\newenvironment{DoxyParams}[2][]{%\n"
+       "  \\begin{DoxyDesc}{#2}%\n"
+       "    \\begin{description}%\n"
+       "      \\item[] \\hspace{\\fill} \\vspace{-25pt}%\n"
+       "      \\definecolor{tableShade}{HTML}{F8F8F8}%\n"
+       "      \\rowcolors{1}{white}{tableShade}%\n"
+       "      \\arrayrulecolor{gray}%\n"
+       "      \\setlength{\\tabcolsep}{0.01\\textwidth}%\n"
+       "      \\ifthenelse{\\equal{#1}{}}\n" // default: name, docs columns
+       "      {\\begin{longtable}{|>{\\raggedleft\\hspace{0pt}}p{0.15\\textwidth}|%\n"
+       "                          p{0.87\\textwidth}|}}%\n"
+       "      {\\ifthenelse{\\equal{#1}{1}}%\n" // inout, name, docs columns, or type, name, docs columns
+       "       {\\begin{longtable}{|>{\\centering}p{0.10\\textwidth}|%\n"
+       "                          >{\\raggedleft\\hspace{0pt}}p{0.15\\textwidth}|%\n"
+       "                          p{0.75\\textwidth}|}}%\n"
+       "       {\\begin{longtable}{|>{\\centering}p{0.10\\textwidth}|%\n" // inout, type, name, docs columns
+       "                          >{\\centering\\hspace{0pt}}p{0.15\\textwidth}|%\n"
+       "                          >{\\raggedleft\\hspace{0pt}}p{0.15\\textwidth}|%\n"
+       "                          p{0.58\\textwidth}|}}%\n"
+       "       }\\hline%\n"
+       "}{%\n"
+       "      \\end{longtable}%\n"
+       "    \\end{description}%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% is used for parameters within a detailed function description\n"
+       "\\newenvironment{DoxyParamCaption}{%\n"
+       "  \\renewcommand{\\item}[2][]{##1 {\\em ##2}}%\n"
+       "  }{%\n"
+       "}\n\n";
+  t << "% Used by return value lists\n"
+       "\\newenvironment{DoxyRetVals}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "    \\begin{description}%\n"
+       "      \\item[] \\hspace{\\fill} \\vspace{-25pt}%\n"
+       "      \\definecolor{tableShade}{HTML}{F8F8F8}%\n"
+       "      \\rowcolors{1}{white}{tableShade}%\n"
+       "      \\arrayrulecolor{gray}%\n"
+       "      \\setlength{\\tabcolsep}{0.01\\textwidth}%\n"
+       "      \\begin{longtable}{|>{\\raggedleft\\hspace{0pt}}p{0.25\\textwidth}|%\n"
+       "                          p{0.77\\textwidth}|}%\n"
+       "      \\hline%\n"
+       "}{%\n"
+       "      \\end{longtable}%\n"
+       "    \\end{description}%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by exception lists\n"
+       "\\newenvironment{DoxyExceptions}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "    \\begin{description}%\n"
+       "      \\item[] \\hspace{\\fill} \\vspace{-25pt}%\n"
+       "      \\definecolor{tableShade}{HTML}{F8F8F8}%\n"
+       "      \\rowcolors{1}{white}{tableShade}%\n"
+       "      \\arrayrulecolor{gray}%\n"
+       "      \\setlength{\\tabcolsep}{0.01\\textwidth}%\n"
+       "      \\begin{longtable}{|>{\\raggedleft\\hspace{0pt}}p{0.25\\textwidth}|%\n"
+       "                          p{0.77\\textwidth}|}%\n"
+       "      \\hline%\n"
+       "}{%\n"
+       "      \\end{longtable}%\n"
+       "    \\end{description}%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
+  t << "% Used by template parameter lists\n"
+       "\\newenvironment{DoxyTemplParams}[1]{%\n"
+       "  \\begin{DoxyDesc}{#1}%\n"
+       "    \\begin{description}%\n"
+       "      \\item[] \\hspace{\\fill} \\vspace{-25pt}%\n"
+       "      \\definecolor{tableShade}{HTML}{F8F8F8}%\n"
+       "      \\rowcolors{1}{white}{tableShade}%\n"
+       "      \\arrayrulecolor{gray}%\n"
+       "      \\setlength{\\tabcolsep}{0.01\\textwidth}%\n"
+       "      \\begin{longtable}{|>{\\raggedleft\\hspace{0pt}}p{0.25\\textwidth}|%\n"
+       "                          p{0.77\\textwidth}|}%\n"
+       "      \\hline%\n"
+       "}{%\n"
+       "      \\end{longtable}%\n"
+       "    \\end{description}%\n"
+       "  \\end{DoxyDesc}%\n"
+       "}\n\n";
   t << "\\newcommand{\\doxyref}[3]{\\textbf{#1} (\\textnormal{#2}\\,\\pageref{#3})}\n";
-  t << "\\newenvironment{DocInclude}\n";
-  t << "{\\footnotesize}\n";
-  t << "{\\normalsize}\n";
-  t << "\\newenvironment{VerbInclude}\n";
-  t << "{\\footnotesize}\n";
-  t << "{\\normalsize}\n";
-  t << "\\newenvironment{Image}\n";
-  t << "{\\begin{figure}[H]}\n";
-  t << "{\\end{figure}}\n";
-  t << "\\newenvironment{ImageNoCaption}{}{}\n";
-  t << "\\newenvironment{CompactList}\n";
+  t << "\\newenvironment{DoxyCompactList}\n";
   t << "{\\begin{list}{}{\n";
   t << "  \\setlength{\\leftmargin}{0.5cm}\n";
   t << "  \\setlength{\\itemsep}{0pt}\n";
@@ -351,7 +701,7 @@ static void writeDefaultStyleSheetPart3(QTextStream &t)
   t << "  \\setlength{\\topsep}{0pt}\n";
   t << "  \\renewcommand{\\makelabel}{\\hfill}}}\n";
   t << "{\\end{list}}\n";
-  t << "\\newenvironment{CompactItemize}\n";
+  t << "\\newenvironment{DoxyCompactItemize}\n";
   t << "{\n";
   t << "  \\begin{itemize}\n";
   t << "  \\setlength{\\itemsep}{-3pt}\n";
@@ -392,7 +742,8 @@ static void writeDefaultStyleSheetPart3(QTextStream &t)
   t << "\\setlength{\\parindent}{0cm}\n";
   t << "\\setlength{\\parskip}{0.2cm}\n";
   t << "\\addtocounter{secnumdepth}{1}\n";
-  t << "\\sloppy\n";
+  // \sloppy should not be used, see bug 563698 
+  //t << "\\sloppy\n";
   t << "\\usepackage[T1]{fontenc}\n";
   t << "\\makeatletter\n";
   t << "\\renewcommand{\\paragraph}{\\@startsection{paragraph}{4}{0ex}%\n";
@@ -412,11 +763,12 @@ static void writeDefaultStyleSheetPart3(QTextStream &t)
   t << "\\definecolor{vhdldigit}{rgb}{1.0,0.0,1.0}\n";
   t << "\\definecolor{vhdlkeyword}{rgb}{0.43,0.0,0.43}\n";
   t << "\\definecolor{vhdllogic}{rgb}{1.0,0.0,0.0}\n";
+  t << "\\definecolor{vhdlchar}{rgb}{0.0,0.0,0.0}\n";
 }
 
 void LatexGenerator::writeHeaderFile(QFile &f)
 {
-  QTextStream t(&f);
+  FTextStream t(&f);
   writeDefaultHeaderPart1(t);
   t << "Your title here";
   writeDefaultHeaderPart2(t);
@@ -426,8 +778,7 @@ void LatexGenerator::writeHeaderFile(QFile &f)
 
 void LatexGenerator::writeStyleSheetFile(QFile &f)
 {
-  QTextStream t(&f);
-  t.setEncoding(QTextStream::UnicodeUTF8);
+  FTextStream t(&f);
 
   writeDefaultStyleSheetPart1(t);
   QCString &projectName = Config_getString("PROJECT_NAME");
@@ -435,12 +786,12 @@ void LatexGenerator::writeStyleSheetFile(QFile &f)
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2008";
+  //t << "Dimitri van Heesch \\copyright~1997-2010";
   writeDefaultStyleSheetPart2(t);
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2008";
+  //t << "Dimitri van Heesch \\copyright~1997-2010";
   writeDefaultStyleSheetPart3(t);
 }
 
@@ -643,7 +994,7 @@ void LatexGenerator::startIndexSection(IndexSections is)
 
 void LatexGenerator::endIndexSection(IndexSections is)
 {
-  static bool compactLatex = Config_getBool("COMPACT_LATEX");
+  //static bool compactLatex = Config_getBool("COMPACT_LATEX");
   static bool sourceBrowser = Config_getBool("SOURCE_BROWSER");
   static QCString latexHeader = Config_getString("LATEX_HEADER");
   switch (is)
@@ -658,7 +1009,7 @@ void LatexGenerator::endIndexSection(IndexSections is)
       break;
     case isMainPage:
       {
-        QCString indexName=usingTreeIndex()?"main":"index";
+        QCString indexName=Config_getBool("GENERATE_TREEVIEW")?"main":"index";
         t << "}\n\\label{index}";
         if (Config_getBool("PDF_HYPERLINKS")) t << "\\hypertarget{index}{}";
         t << "\\input{" << indexName << "}\n";
@@ -702,7 +1053,8 @@ void LatexGenerator::endIndexSection(IndexSections is)
         {
           if (!gd->isReference())
           {
-            if (compactLatex) t << "\\input"; else t << "\\include";
+            //if (compactLatex) t << "\\input"; else t << "\\include";
+            t << "\\input"; 
             t << "{" << gd->getOutputFileBase() << "}\n";
           }
         }
@@ -725,7 +1077,8 @@ void LatexGenerator::endIndexSection(IndexSections is)
         {
           if (dd->isLinkableInProject())
           {
-            if (compactLatex) t << "\\input"; else t << "\\include";
+            //if (compactLatex) t << "\\input"; else t << "\\include";
+            t << "\\input"; 
             t << "{" << dd->getOutputFileBase() << "}\n";
           }
         }
@@ -748,7 +1101,8 @@ void LatexGenerator::endIndexSection(IndexSections is)
         {
           if (nd->isLinkableInProject())
           {
-            if (compactLatex) t << "\\input"; else t << "\\include";
+            //if (compactLatex) t << "\\input"; else t << "\\include";
+            t << "\\input"; 
             t << "{" << nd->getOutputFileBase() << "}\n";
           }
           ++nli;
@@ -772,7 +1126,8 @@ void LatexGenerator::endIndexSection(IndexSections is)
         {
           if (cd->isLinkableInProject() && cd->templateMaster()==0)
           {
-            if (compactLatex) t << "\\input"; else t << "\\include";
+            //if (compactLatex) t << "\\input"; else t << "\\include";
+            t << "\\input"; 
             t << "{" << cd->getOutputFileBase() << "}\n";
           } 
         }
@@ -792,20 +1147,22 @@ void LatexGenerator::endIndexSection(IndexSections is)
               if (isFirst)
               {
                 t << "}\n\\input{" << fd->getOutputFileBase() << "}\n";
-                if (sourceBrowser && m_prettyCode)
+                if (sourceBrowser && m_prettyCode && fd->generateSourceFile())
                 {
+                  //t << "\\include{" << fd->getSourceFileBase() << "}\n";
                   t << "\\input{" << fd->getSourceFileBase() << "}\n";
                 }
                 isFirst=FALSE;
               }
               else
               {
-                if (compactLatex) t << "\\input" ; else t << "\\include";
+                //if (compactLatex) t << "\\input" ; else t << "\\include";
+                t << "\\input" ; 
                 t << "{" << fd->getOutputFileBase() << "}\n";
-                if (sourceBrowser && m_prettyCode)
+                if (sourceBrowser && m_prettyCode && fd->generateSourceFile())
                 {
-                  if (compactLatex) t << "\\input" ; else t << "\\include";
-                  t << "{" << fd->getSourceFileBase() << "}\n";
+                  //t << "\\include{" << fd->getSourceFileBase() << "}\n";
+                  t << "\\input{" << fd->getSourceFileBase() << "}\n";
                 }
               }
             }
@@ -826,7 +1183,8 @@ void LatexGenerator::endIndexSection(IndexSections is)
         }
         for (++pdi;(pd=pdi.current());++pdi)
         {
-          if (compactLatex) t << "\\input" ; else t << "\\include";
+          //if (compactLatex) t << "\\input" ; else t << "\\include";
+          t << "\\input"; 
           t << "{" << pd->getOutputFileBase() << "}\n";
         }
       }
@@ -863,10 +1221,12 @@ void LatexGenerator::endIndexSection(IndexSections is)
   }
 }
 
-void LatexGenerator::writePageLink(const char *name, bool first)
+void LatexGenerator::writePageLink(const char *name, bool /*first*/)
 {
-  bool &compactLatex = Config_getBool("COMPACT_LATEX");
-  if (compactLatex || first) t << "\\input" ; else t << "\\include";
+  //bool &compactLatex = Config_getBool("COMPACT_LATEX");
+  // next is remove for bug615957
+  //if (compactLatex || first) t << "\\input" ; else t << "\\include";
+  t << "\\input" ; 
   t << "{" << name << "}\n";
 }
 
@@ -888,14 +1248,11 @@ void LatexGenerator::writeStyleInfo(int part)
       break;
     case 2:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2008";
-        t << "}]{}\n";
         writeDefaultStyleSheetPart2(t);
       }
       break;
     case 4:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2008";
         writeDefaultStyleSheetPart3(t);
         endPlainFile();
       }
@@ -915,6 +1272,7 @@ void LatexGenerator::startParagraph()
 
 void LatexGenerator::endParagraph()
 {
+  t << endl << endl;
 }
 
 void LatexGenerator::writeString(const char *text)
@@ -1168,7 +1526,7 @@ void LatexGenerator::endGroupHeader()
   t << "}" << endl;
 }
 
-void LatexGenerator::startMemberHeader()
+void LatexGenerator::startMemberHeader(const char *)
 {
   if (Config_getBool("COMPACT_LATEX")) 
   {
@@ -1360,7 +1718,6 @@ void LatexGenerator::endSection(const char *lab,SectionInfo::SectionType)
 }
 
 
-//void LatexGenerator::docifyStatic(QTextStream &t,const char *str)
 void LatexGenerator::docify(const char *str)
 {
   filterLatexString(t,str,insideTabbing,FALSE);
@@ -1368,38 +1725,18 @@ void LatexGenerator::docify(const char *str)
 
 void LatexGenerator::codify(const char *str)
 {
-  static bool isJapanese  = theTranslator->idLanguage()=="japanese" || 
-                            theTranslator->idLanguage()=="japanese-en";
-
   if (str)
   { 
     const char *p=str;
     char c;
-    char cs[2];
-    cs[1]='\0';
+    char cs[5];
     int spacesToNextTabStop;
     static int tabSize = Config_getInt("TAB_SIZE");
     while (*p)
     {
-      static bool MultiByte = FALSE;
+      //static bool MultiByte = FALSE;
       c=*p++;
 
-      if( isJapanese )
-      {
-        if ( MultiByte )
-        {
-          t << (char)c;
-          MultiByte = FALSE;
-          continue;
-        }
-        if ((uchar)c>=0x80) // char in range [0x80..0xff]
-        {
-          t << (char)c;
-          MultiByte = TRUE;
-          continue;
-        }
-      }
-
       switch(c)
       {
         case 0x0c: break; // remove ^L
@@ -1409,14 +1746,34 @@ void LatexGenerator::codify(const char *str)
                    col+=spacesToNextTabStop;
                    break; 
         case '\n': t << '\n'; col=0;                    break;
-        default:   if (m_prettyCode)
+        default:   cs[0]=c;
+                   cs[1]=0;
+                   int bytes=1;
+                   if (c<0) // multibyte utf-8 character
+                   {
+                     bytes++;   // 1xxx.xxxx: >=2 byte character
+                     cs[1]=*p;
+                     cs[2]=0;
+                     if (((uchar)c&0xE0)==0xE0)
+                     {
+                       bytes++; // 111x.xxxx: >=3 byte character
+                       cs[2]=*(p+1);
+                       cs[3]=0;
+                     }
+                     if (((uchar)c&0xF0)==0xF0)
+                     {
+                       bytes++; // 1111.xxxx: 4 byte character
+                       cs[2]=*(p+2);
+                       cs[4]=0;
+                     }
+                   }
+                   if (m_prettyCode)
                    {
-                     cs[0]=c; 
                      filterLatexString(t,cs,insideTabbing,TRUE);
                    }
                    else
                    {
-                     t << c;
+                     t << cs;
                    }
                    if (col>=80)
                    {
@@ -1425,8 +1782,9 @@ void LatexGenerator::codify(const char *str)
                    }
                    else
                    {
-                     col++;                    
+                     col++;
                    }
+                   p+=(bytes-1); // skip to next character
                    break;
       }
     }
@@ -1515,7 +1873,7 @@ void LatexGenerator::startMemberDescription()
 {
   if (!insideTabbing)
   { 
-    t << "\\begin{CompactList}\\small\\item\\em "; 
+    t << "\\begin{DoxyCompactList}\\small\\item\\em "; 
   }
   else
   {
@@ -1528,7 +1886,7 @@ void LatexGenerator::endMemberDescription()
 { 
   if (!insideTabbing)
   {
-    t << "\\item\\end{CompactList}"; 
+    t << "\\item\\end{DoxyCompactList}"; 
   }
   else
   {
@@ -1552,7 +1910,7 @@ void LatexGenerator::startMemberList()
 { 
   if (!insideTabbing)
   {
-    t << "\\begin{CompactItemize}" << endl; 
+    t << "\\begin{DoxyCompactItemize}" << endl; 
   }
 }
 
@@ -1561,7 +1919,7 @@ void LatexGenerator::endMemberList()
   //printf("LatexGenerator::endMemberList(%d)\n",insideTabbing);
   if (!insideTabbing)
   {
-    t << "\\end{CompactItemize}"   << endl; 
+    t << "\\end{DoxyCompactItemize}"   << endl; 
   }
 }
 
@@ -1599,11 +1957,12 @@ void LatexGenerator::endMemberGroup(bool hasHeader)
 
 void LatexGenerator::startDotGraph() 
 {
+  newParagraph();
 }
 
 void LatexGenerator::endDotGraph(const DotClassGraph &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),fileName,relPath);
 }
 
 void LatexGenerator::startInclDepGraph() 
@@ -1612,7 +1971,7 @@ void LatexGenerator::startInclDepGraph()
 
 void LatexGenerator::endInclDepGraph(const DotInclDepGraph &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),fileName,relPath);
 }
 
 void LatexGenerator::startGroupCollaboration() 
@@ -1621,7 +1980,7 @@ void LatexGenerator::startGroupCollaboration()
 
 void LatexGenerator::endGroupCollaboration(const DotGroupCollaboration &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),fileName,relPath);
 }
 
 void LatexGenerator::startCallGraph() 
@@ -1630,7 +1989,7 @@ void LatexGenerator::startCallGraph()
 
 void LatexGenerator::endCallGraph(const DotCallGraph &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),fileName,relPath);
 }
 
 void LatexGenerator::startDirDepGraph() 
@@ -1639,7 +1998,7 @@ void LatexGenerator::startDirDepGraph()
 
 void LatexGenerator::endDirDepGraph(const DotDirDeps &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),fileName,relPath);
 }
 
 void LatexGenerator::startDescription() 
@@ -1704,14 +2063,50 @@ void LatexGenerator::endParamList()
   t << "\\end{Desc}" << endl;
 }
 
-void LatexGenerator::startParameterType(bool first,const char *key)
+void LatexGenerator::startParameterList(bool openBracket)
 {
-  if (!first)
+  /* start of ParameterType ParameterName list */
+  if (openBracket) t << "(";
+  t << endl << "\\begin{DoxyParamCaption}" << endl;
+}
+
+void LatexGenerator::endParameterList()
+{
+}
+
+
+void LatexGenerator::startParameterType(bool /*first*/,const char *key)
+{
+  t << "\\item[{";
+  t << key;
+//  if (!first)
+//  {
+//    t << "\\/ " << key << " ";
+//  }
+}
+
+void LatexGenerator::endParameterType()
+{
+  t << "}]";
+}
+
+void LatexGenerator::startParameterName(bool /*oneArgOnly*/)
+{
+  t << "{";
+}
+
+void LatexGenerator::endParameterName(bool last,bool /* emptyList */,bool closeBracket)
+{
+  t << "}" << endl;
+
+  if (last)
   {
-    t << "\\/ " << key << " ";
+    t << "\\end{DoxyParamCaption}" << endl;
+    if (closeBracket) t << ")";
   }
 }
 
+
 void LatexGenerator::printDoc(DocNode *n,const char *langExt)
 {
   LatexDocVisitor *visitor = new LatexDocVisitor(t,*this,langExt,insideTabbing);
@@ -1801,27 +2196,27 @@ void LatexGenerator::escapeMakeIndexChars(const char *s)
 
 void LatexGenerator::startCodeFragment()
 {
-  if (m_prettyCode)
-  {
-    t << endl << endl;
-    t << "\\begin{footnotesize}\\begin{alltt}\n";
-  }
-  else
-  {
-    t << endl << endl << "\\begin{Code}\\begin{verbatim}";
-  }
+  //if (m_prettyCode)
+  //{
+  //  t << endl << endl;
+  //  t << "\\begin{footnotesize}\\begin{alltt}\n";
+  //}
+  //else
+  //{
+    t << "\n\\begin{DoxyCode}\n";
+  //}
 }
 
 void LatexGenerator::endCodeFragment()
 {
-  if (m_prettyCode)
-  {
-    t << "\\end{alltt}\\end{footnotesize}" << endl; 
-  }
-  else
-  {
-    t << "\\end{verbatim}\n\\end{Code}" << endl;
-  }
+  //if (m_prettyCode)
+  //{
+  //  t << "\\end{alltt}\\end{footnotesize}" << endl; 
+  //}
+  //else
+  //{
+    t << "\\end{DoxyCode}\n";
+  //}
 }
 
 void LatexGenerator::writeLineNumber(const char *ref,const char *fileName,const char *anchor,int l)
