@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -26,10 +26,12 @@
 #include "util.h"
 #include "message.h"
 #include <qfileinfo.h> 
+#include "parserintf.h"
 
-ManDocVisitor::ManDocVisitor(QTextStream &t,BaseCodeDocInterface &ci) 
+ManDocVisitor::ManDocVisitor(FTextStream &t,CodeOutputInterface &ci,
+                             const char *langExt) 
   : DocVisitor(DocVisitor_Man), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE), m_firstCol(TRUE),
-    m_indent(0)
+    m_indent(0), m_langExt(langExt)
 {
 }
 
@@ -85,7 +87,13 @@ void ManDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Tm:      m_t << "(TM)"; break;
     case DocSymbol::Reg:     m_t << "(R)"; break;
     case DocSymbol::Apos:    m_t << "'"; break;
-    case DocSymbol::Quot:    m_t << "''"; break;
+    case DocSymbol::Quot:    m_t << "\""; break;
+    case DocSymbol::Lsquo:   m_t << "`"; break;
+    case DocSymbol::Rsquo:   m_t << "'"; break;
+    case DocSymbol::Ldquo:   m_t << "``"; break;
+    case DocSymbol::Rdquo:   m_t << "''"; break;
+    case DocSymbol::Ndash:   m_t << "--"; break;
+    case DocSymbol::Mdash:   m_t << "---"; break;
     case DocSymbol::Uml:     m_t << s->letter() << "\\*(4"; break;
     case DocSymbol::Acute:   m_t << s->letter() << "\\*(`"; break;
     case DocSymbol::Grave:   m_t << s->letter() << "\\*:"; break;
@@ -97,7 +105,7 @@ void ManDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Ring:    m_t << s->letter() << "\\*o"; break;
     case DocSymbol::Nbsp:    m_t << " "; break;
     default:
-                             err("Error: unknown symbol found\n");
+                             err("error: unknown symbol found\n");
   }
   m_firstCol=FALSE;
 }
@@ -186,7 +194,9 @@ void ManDocVisitor::visit(DocVerbatim *s)
       if (!m_firstCol) m_t << endl;
       m_t << ".PP" << endl;
       m_t << ".nf" << endl;
-      parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
+      Doxygen::parserManager->getParser(0/*TODO*/)
+                            ->parseCode(m_ci,s->context(),s->text(),
+                                        s->isExample(),s->exampleFile());
       if (!m_firstCol) m_t << endl;
       m_t << ".fi" << endl;
       m_t << ".PP" << endl;
@@ -209,6 +219,7 @@ void ManDocVisitor::visit(DocVerbatim *s)
     case DocVerbatim::XmlOnly: 
     case DocVerbatim::LatexOnly: 
     case DocVerbatim::Dot: 
+    case DocVerbatim::Msc: 
       /* nothing */ 
       break;
   }
@@ -231,7 +242,11 @@ void ManDocVisitor::visit(DocInclude *inc)
          m_t << ".nf" << endl;
          QFileInfo cfi( inc->file() );
          FileDef fd( cfi.dirPath(), cfi.fileName() );
-         parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile(), &fd);
+         Doxygen::parserManager->getParser(inc->extension())
+                               ->parseCode(m_ci,inc->context(),
+                                           inc->text(),
+                                           inc->isExample(),
+                                           inc->exampleFile(), &fd);
          if (!m_firstCol) m_t << endl;
          m_t << ".fi" << endl;
          m_t << ".PP" << endl;
@@ -242,7 +257,10 @@ void ManDocVisitor::visit(DocInclude *inc)
       if (!m_firstCol) m_t << endl;
       m_t << ".PP" << endl;
       m_t << ".nf" << endl;
-      parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile());
+      Doxygen::parserManager->getParser(inc->extension())
+                            ->parseCode(m_ci,inc->context(),
+                                        inc->text(),inc->isExample(),
+                                        inc->exampleFile());
       if (!m_firstCol) m_t << endl;
       m_t << ".fi" << endl;
       m_t << ".PP" << endl;
@@ -283,7 +301,12 @@ void ManDocVisitor::visit(DocIncOperator *op)
   if (op->type()!=DocIncOperator::Skip) 
   {
     popEnabled();
-    if (!m_hide) parseCode(m_ci,op->context(),op->text().latin1(),op->isExample(),op->exampleFile());
+    if (!m_hide) 
+    {
+      Doxygen::parserManager->getParser(0/*TODO*/)
+                            ->parseCode(m_ci,op->context(),op->text(),
+                                        op->isExample(),op->exampleFile());
+    }
     pushEnabled();
     m_hide=TRUE;
   }
@@ -314,6 +337,10 @@ void ManDocVisitor::visit(DocIndexEntry *)
 {
 }
 
+void ManDocVisitor::visit(DocSimpleSectSep *)
+{
+}
+
 //--------------------------------------
 // visitor functions for compound nodes
 //--------------------------------------
@@ -452,7 +479,7 @@ void ManDocVisitor::visitPre(DocTitle *)
 void ManDocVisitor::visitPost(DocTitle *)
 {
   if (m_hide) return;
-  m_t << "\\fP";
+  m_t << "\\fP" << endl;
   m_t << ".RS 4" << endl;
 }
 
@@ -691,6 +718,14 @@ void ManDocVisitor::visitPre(DocDotFile *)
 void ManDocVisitor::visitPost(DocDotFile *) 
 {
 }
+void ManDocVisitor::visitPre(DocMscFile *)
+{
+}
+
+void ManDocVisitor::visitPost(DocMscFile *) 
+{
+}
+
 
 void ManDocVisitor::visitPre(DocLink *)
 {
@@ -784,6 +819,11 @@ void ManDocVisitor::visitPre(DocParamSect *s)
       m_t << theTranslator->trReturnValues(); break;
     case DocParamSect::Exception: 
       m_t << theTranslator->trExceptions(); break;
+    case DocParamSect::TemplateParam: 
+      /* TODO: add this 
+      m_t << theTranslator->trTemplateParam(); break;
+      */
+      m_t << "Template Parameters"; break;
     default:
       ASSERT(0);
   }
@@ -804,13 +844,22 @@ void ManDocVisitor::visitPre(DocParamList *pl)
 {
   if (m_hide) return;
   m_t << "\\fI";
-  QStrListIterator li(pl->parameters());
-  const char *s;
+  //QStrListIterator li(pl->parameters());
+  //const char *s;
+  QListIterator<DocNode> li(pl->parameters());
+  DocNode *param;
   bool first=TRUE;
-  for (li.toFirst();(s=li.current());++li)
+  for (li.toFirst();(param=li.current());++li)
   {
     if (!first) m_t << ","; else first=FALSE;
-    m_t << s;
+    if (param->kind()==DocNode::Kind_Word)
+    {
+      visit((DocWord*)param); 
+    }
+    else if (param->kind()==DocNode::Kind_LinkedWord)
+    {
+      visit((DocLinkedWord*)param); 
+    }
   }
   m_t << "\\fP ";
 }
