@@ -2,7 +2,7 @@
  *
  * $Id: membergroup.cpp,v 1.19 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2007 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,6 +28,8 @@
 #include "doxygen.h"
 #include "docparser.h"
 #include "marshal.h"
+#include "entry.h"
+#include "md5.h"
 
 //static QCString idToName(int id)
 //{
@@ -55,6 +57,7 @@ MemberGroup::MemberGroup(Definition *parent,
   m_numDocMembers = -1;
   m_parent        = parent;
   m_docFile       = docFile;
+  m_xrefListItems = 0;
   //printf("Member group docs=`%s'\n",doc.data());
 }
 
@@ -105,7 +108,9 @@ void MemberGroup::writeDeclarations(OutputList &ol,
                ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd)
 {
   //printf("MemberGroup::writeDeclarations() memberList->count()=%d\n",memberList->count());
-  memberList->writeDeclarations(ol,cd,nd,fd,gd,grpHeader,doc/*,TRUE,TRUE*/);
+  QCString ldoc = doc;
+  if (!ldoc.isEmpty()) ldoc.prepend("<a name=\""+anchor()+"\"></a>");
+  memberList->writeDeclarations(ol,cd,nd,fd,gd,grpHeader,ldoc/*,TRUE,TRUE*/);
 }
 
 void MemberGroup::writePlainDeclarations(OutputList &ol,
@@ -165,17 +170,30 @@ void MemberGroup::distributeMemberGroupDocumentation()
 {
   //printf("MemberGroup::distributeMemberGroupDocumentation() %s\n",grpHeader.data());
   MemberDef *md=memberList->first();
-  if (md && !(md->documentation().isEmpty() && 
-              md->briefDescription().isEmpty() && 
-              md->inbodyDocumentation().isEmpty()
-             )
-     )
+  while (md)
   {
-    //printf("First member %s has documentation!\n",md->name().data());
-    MemberDef *omd=memberList->next();
+    //printf("checking md=%s\n",md->name().data());
+    // find the first member of the group with documentation
+    if (!md->documentation().isEmpty()       ||
+        !md->briefDescription().isEmpty()    ||
+        !md->inbodyDocumentation().isEmpty()
+       )
+    {
+      //printf("found it!\n");
+      break;
+    }
+    md=memberList->next();
+  }
+  if (md) // distribute docs of md to other members of the list
+  {
+    //printf("Member %s has documentation!\n",md->name().data());
+    MemberDef *omd=memberList->first();
     while (omd)
     {
-      if (omd->documentation().isEmpty() && omd->briefDescription().isEmpty() && omd->inbodyDocumentation().isEmpty())
+      if (md!=omd && omd->documentation().isEmpty() && 
+                     omd->briefDescription().isEmpty() && 
+                     omd->inbodyDocumentation().isEmpty()
+         )
       {
         //printf("Copying documentation to member %s\n",omd->name().data());
         omd->setBriefDescription(md->briefDescription(),md->briefFile(),md->briefLine());
@@ -187,9 +205,9 @@ void MemberGroup::distributeMemberGroupDocumentation()
   }
 }
 
-int MemberGroup::varCount() const       
-{ 
-  return memberList->varCount(); 
+int MemberGroup::varCount() const
+{
+  return memberList->varCount();
 }
 
 int MemberGroup::funcCount() const      
@@ -242,9 +260,29 @@ void MemberGroup::setInGroup(bool b)
   memberList->setInGroup(b);
 }
 
+QCString MemberGroup::anchor() const
+{
+  uchar md5_sig[16];
+  QCString sigStr(33);
+  QCString locHeader = grpHeader;
+  if (locHeader.isEmpty()) locHeader="[NOHEADER]";
+  MD5Buffer((const unsigned char *)locHeader.data(),locHeader.length(),md5_sig);
+  MD5SigToString(md5_sig,sigStr.data(),33);
+  return "amgrp"+sigStr;
+}
+
 void MemberGroup::addListReferences(Definition *def)
 {
   memberList->addListReferences(def);
+  if (m_xrefListItems && def)
+  {
+    QCString name = def->getOutputFileBase()+"#"+anchor();
+    addRefItem(m_xrefListItems,
+        name,
+        theTranslator->trGroup(TRUE,TRUE),
+        name,
+        grpHeader,0);
+  }
 }
 
 void MemberGroup::findSectionsInDocumentation()
@@ -267,6 +305,7 @@ void MemberGroup::marshal(StorageIntf *s)
   marshalInt(s,m_numDocMembers);
   marshalObjPointer(s,m_parent);
   marshalQCString(s,m_docFile);
+  marshalItemInfoList (Doxygen::symbolStorage,m_xrefListItems);
 }
 
 void MemberGroup::unmarshal(StorageIntf *s)
@@ -283,5 +322,41 @@ void MemberGroup::unmarshal(StorageIntf *s)
   m_numDocMembers = unmarshalInt(s);
   m_parent        = (Definition *)unmarshalObjPointer(s);
   m_docFile       = unmarshalQCString(s);
+  m_xrefListItems = unmarshalItemInfoList (Doxygen::symbolStorage);
 }
 
+void MemberGroup::setRefItems(const QList<ListItemInfo> *sli)
+{
+  if (sli)
+  {
+    // deep copy the list
+    if (m_xrefListItems==0) 
+    {
+      m_xrefListItems=new QList<ListItemInfo>;
+      m_xrefListItems->setAutoDelete(TRUE);
+    }
+    QListIterator<ListItemInfo> slii(*sli);
+    ListItemInfo *lii;
+    for (slii.toFirst();(lii=slii.current());++slii)
+    {
+      m_xrefListItems->append(new ListItemInfo(*lii));
+    } 
+  }
+}
+//--------------------------------------------------------------------------
+
+void MemberGroupInfo::setRefItems(const QList<ListItemInfo> *sli)
+{
+  if (!sli) return;
+  if (m_sli==0)
+  {
+    m_sli = new QList<ListItemInfo>;
+    m_sli->setAutoDelete(TRUE);
+  }
+  QListIterator<ListItemInfo> slii(*sli);
+  ListItemInfo *ili;
+  for (slii.toFirst();(ili=slii.current());++slii)
+  {
+    m_sli->append(new ListItemInfo(*ili));
+  }
+}
