@@ -2,7 +2,7 @@
  *
  * $Id: util.h,v 1.49 2001/03/19 19:27:42 root Exp $
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -53,6 +53,10 @@ class PageDef;
 struct SectionInfo;
 class QDir;
 class Definition;
+class BufStr;
+class QFileInfo;
+class QStrList;
+class FTextStream;
 
 //--------------------------------------------------------------------
 
@@ -85,17 +89,19 @@ class TextGeneratorOLImpl : public TextGeneratorIntf
 
 enum SrcLangExt
 {
-  SrcLangExt_IDL    = 0x0008,
-  SrcLangExt_Java   = 0x0010,
-  SrcLangExt_CSharp = 0x0020,
-  SrcLangExt_D      = 0x0040,
-  SrcLangExt_PHP    = 0x0080,
-  SrcLangExt_ObjC   = 0x0100,
-  SrcLangExt_Cpp    = 0x0200,
-  SrcLangExt_JS     = 0x0400,
-  SrcLangExt_Python = 0x0800,
-  SrcLangExt_F90    = 0x1000,
-  SrcLangExt_VHDL   = 0x2000
+  SrcLangExt_Unknown = 0x0000,
+  SrcLangExt_IDL     = 0x0008,
+  SrcLangExt_Java    = 0x0010,
+  SrcLangExt_CSharp  = 0x0020,
+  SrcLangExt_D       = 0x0040,
+  SrcLangExt_PHP     = 0x0080,
+  SrcLangExt_ObjC    = 0x0100,
+  SrcLangExt_Cpp     = 0x0200,
+  SrcLangExt_JS      = 0x0400,
+  SrcLangExt_Python  = 0x0800,
+  SrcLangExt_F90     = 0x1000,
+  SrcLangExt_VHDL    = 0x2000,
+  SrcLangExt_XML     = 0x4000
 };
 
 //--------------------------------------------------------------------
@@ -137,7 +143,8 @@ bool resolveRef(/* in */  const char *scName,
                 /* out */ Definition **resContext,
                 /* out */ MemberDef  **resMember,
                 /* in */  bool lookForSpecializations = TRUE,
-                /* in */  FileDef *currentFile = 0
+                /* in */  FileDef *currentFile = 0,
+                /* in */  bool checkScope = FALSE
                );
 
 bool resolveLink(/* in */  const char *scName,
@@ -195,7 +202,7 @@ int guessSection(const char *name);
 
 inline bool isId(int c)
 {
-  return c=='_' || isalnum(c) || c>=128 || c<0;
+  return c=='_' || c>=128 || c<0 || isalnum(c);
 }
 
 QCString removeRedundantWhiteSpace(const QCString &s);
@@ -232,7 +239,7 @@ bool hasVisibleRoot(BaseClassList *bcl);
 
 int minClassDistance(const ClassDef *cd,const ClassDef *bcd,int level=0);
 
-QCString convertNameToFile(const char *name,bool allowDots=FALSE);
+QCString convertNameToFile(const char *name,bool allowDots=FALSE,bool allowUnderscore=FALSE);
 
 void extractNamespaceName(const QCString &scopeName,
                           QCString &className,QCString &namespaceName,
@@ -278,7 +285,8 @@ int getScopeFragment(const QCString &s,int p,int *l);
 int filterCRLF(char *buf,int len);
 
 void addRefItem(const QList<ListItemInfo> *sli,const char *prefix,
-                        const char *name,const char *title,const char *args=0);
+                const char *key,
+                const char *name,const char *title,const char *args);
 
 PageDef *addRelatedPage(const char *name,const QCString &ptitle,
                            const QCString &doc,QList<SectionInfo> *anchors,
@@ -288,11 +296,11 @@ PageDef *addRelatedPage(const char *name,const QCString &ptitle,
                            TagInfo *tagInfo=0
                           );
 
-QCString escapeCharsInString(const char *name,bool allowDots);
+QCString escapeCharsInString(const char *name,bool allowDots,bool allowUnderscore=FALSE);
 
 void addGroupListToTitle(OutputList &ol,Definition *d);
 
-void filterLatexString(QTextStream &t,const char *str,
+void filterLatexString(FTextStream &t,const char *str,
                        bool insideTabbing=FALSE,bool insidePre=FALSE,
                        bool insideItem=FALSE);
 
@@ -325,9 +333,9 @@ bool findAndRemoveWord(QCString &s,const QCString &word);
 
 QCString stripLeadingAndTrailingEmptyLines(const QCString &s);
 
-void stringToSearchIndex(const QCString &docUrlBase,const QCString &title,
-                         const QCString &str, bool priority=FALSE,
-                         const QCString &anchor="");
+//void stringToSearchIndex(const QCString &docUrlBase,const QCString &title,
+//                         const QCString &str, bool priority=FALSE,
+//                         const QCString &anchor="");
 
 bool updateLanguageMapping(const QCString &extension,const QCString &parser);
 SrcLangExt getLanguageFromFileName(const QCString fileName);
@@ -340,7 +348,7 @@ ClassDef *newResolveTypedef(FileDef *fileScope,MemberDef *md,
                             QCString *pResolvedType=0,
                             ArgumentList *actTemplParams=0);
 
-QCString parseCommentAsText(const Definition *scope,const MemberDef *member,const QString &doc,const QCString &fileName,int lineNr);
+QCString parseCommentAsText(const Definition *scope,const MemberDef *member,const QCString &doc,const QCString &fileName,int lineNr);
 
 QCString transcodeCharacterStringToUTF8(const QCString &input);
 
@@ -362,5 +370,17 @@ QCString convertCharEntitiesToUTF8(const QCString &s);
 bool usingTreeIndex();
 void stackTrace();
 
+bool readInputFile(const char *fileName,BufStr &inBuf);
+QCString filterTitle(const QCString &title);
+
+bool patternMatch(const QFileInfo &fi,const QStrList *patList);
+
+void writeSummaryLink(OutputList &ol,const char *label,const char *title,
+                      bool &first);
+
+QCString externalLinkTarget();
+QCString externalRef(const QCString &relPath,const QCString &ref,bool href);
+int nextUtf8CharPosition(const QCString &utf8Str,int len,int startPos);
+
 #endif
 
