@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2008 by Dimitri van Heesch.
+ * Copyright (C) 1997-2010 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,7 +28,7 @@
 #include <qfileinfo.h> 
 #include "parserintf.h"
 
-XmlDocVisitor::XmlDocVisitor(QTextStream &t,CodeOutputInterface &ci) 
+XmlDocVisitor::XmlDocVisitor(FTextStream &t,CodeOutputInterface &ci) 
   : DocVisitor(DocVisitor_XML), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
 {
 }
@@ -101,7 +101,7 @@ void XmlDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Aelig:   m_t << "<aelig/>"; break;
     case DocSymbol::AElig:   m_t << "<AElig/>"; break;
     default:
-                             err("Error: unknown symbol found\n");
+                             err("error: unknown symbol found\n");
   }
 }
 
@@ -179,7 +179,7 @@ void XmlDocVisitor::visit(DocVerbatim *s)
     case DocVerbatim::Code: // fall though
       m_t << "<programlisting>"; 
       Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,s->context(),s->text().latin1(),
+                            ->parseCode(m_ci,s->context(),s->text(),
                                         s->isExample(),s->exampleFile());
       m_t << "</programlisting>"; 
       break;
@@ -237,7 +237,7 @@ void XmlDocVisitor::visit(DocInclude *inc)
          FileDef fd( cfi.dirPath(), cfi.fileName() );
          Doxygen::parserManager->getParser(inc->extension())
                                ->parseCode(m_ci,inc->context(),
-                                           inc->text().latin1(),
+                                           inc->text(),
                                            inc->isExample(),
                                            inc->exampleFile(), &fd);
          m_t << "</programlisting>"; 
@@ -247,7 +247,7 @@ void XmlDocVisitor::visit(DocInclude *inc)
       m_t << "<programlisting>";
       Doxygen::parserManager->getParser(inc->extension())
                             ->parseCode(m_ci,inc->context(),
-                                        inc->text().latin1(),
+                                        inc->text(),
                                         inc->isExample(),
                                         inc->exampleFile());
       m_t << "</programlisting>"; 
@@ -287,7 +287,7 @@ void XmlDocVisitor::visit(DocIncOperator *op)
     {
       Doxygen::parserManager->getParser(m_langExt)
                             ->parseCode(m_ci,op->context(),
-                                        op->text().latin1(),op->isExample(),
+                                        op->text(),op->isExample(),
                                         op->exampleFile());
     }
     pushEnabled();
@@ -323,6 +323,11 @@ void XmlDocVisitor::visit(DocIndexEntry *ie)
          "</indexentry>";
 }
 
+void XmlDocVisitor::visit(DocSimpleSectSep *)
+{
+  m_t << "<simplesectsep/>";
+}
+
 //--------------------------------------
 // visitor functions for compound nodes
 //--------------------------------------
@@ -651,7 +656,7 @@ void XmlDocVisitor::visitPre(DocImage *img)
   }
   m_t << "\"";
 
-  QString baseName=img->name();
+  QCString baseName=img->name();
   int i;
   if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
   {
@@ -674,7 +679,7 @@ void XmlDocVisitor::visitPre(DocImage *img)
 
   // copy the image to the output dir
   QFile inImage(img->name());
-  QFile outImage(Config_getString("XML_OUTPUT")+"/"+baseName.ascii());
+  QFile outImage(Config_getString("XML_OUTPUT")+"/"+baseName.data());
   if (inImage.open(IO_ReadOnly))
   {
     if (outImage.open(IO_WriteOnly))
@@ -683,7 +688,7 @@ void XmlDocVisitor::visitPre(DocImage *img)
       inImage.readBlock(buffer,inImage.size());
       outImage.writeBlock(buffer,inImage.size());
       outImage.flush();
-      delete buffer;
+      delete[] buffer;
     }
   }
 }
@@ -706,6 +711,17 @@ void XmlDocVisitor::visitPost(DocDotFile *)
   m_t << "</dotfile>" << endl;
 }
 
+void XmlDocVisitor::visitPre(DocMscFile *df)
+{
+  if (m_hide) return;
+  m_t << "<mscfile name=\"" << df->file() << "\">";
+}
+
+void XmlDocVisitor::visitPost(DocMscFile *) 
+{
+  if (m_hide) return;
+  m_t << "</dotfile>" << endl;
+}
 void XmlDocVisitor::visitPre(DocLink *lnk)
 {
   if (m_hide) return;
@@ -805,6 +821,24 @@ void XmlDocVisitor::visitPre(DocParamList *pl)
   DocNode *param;
   for (li.toFirst();(param=li.current());++li)
   {
+    if (pl->paramTypes().count()>0)
+    {
+      QListIterator<DocNode> li(pl->paramTypes());
+      DocNode *type;
+      for (li.toFirst();(type=li.current());++li)
+      {
+        m_t << "<parametertype>";
+        if (type->kind()==DocNode::Kind_Word)
+        {
+          visit((DocWord*)type); 
+        }
+        else if (type->kind()==DocNode::Kind_LinkedWord)
+        {
+          visit((DocLinkedWord*)type); 
+        }
+        m_t << "</parametertype>" << endl;
+      }
+    }
     m_t << "<parametername";
     if (pl->direction()!=DocParamSect::Unspecified)
     {
@@ -902,7 +936,7 @@ void XmlDocVisitor::filter(const char *str)
   m_t << convertToXML(str);
 }
 
-void XmlDocVisitor::startLink(const QString &ref,const QString &file,const QString &anchor)
+void XmlDocVisitor::startLink(const QCString &ref,const QCString &file,const QCString &anchor)
 {
   m_t << "<ref refid=\"" << file;
   if (!anchor.isEmpty()) m_t << "_1" << anchor;
